@@ -362,7 +362,7 @@ void PptToOdp::DrawClient::addTextStyles(
     if (isPlaceholder) {
         out.xml.addAttribute("presentation:style-name", styleName);
         QString className = getPresentationClass(cd->placeholderAtom.data());
-        if (className.isEmpty()) {
+        if (className.isEmpty() || !out.stylesxml) {
             const TextContainer* tc = ppttoodp->getTextContainer(tb, cd);
             className = getPresentationClass(tc);
             out.xml.addAttribute("presentation:placeholder", "false");
@@ -1623,6 +1623,10 @@ QPair<QString, QString> PptToOdp::findHyperlink(const quint32 id)
 {
     QString friendly;
     QString target;
+
+    if( !p->documentContainer->exObjList )
+        return qMakePair(friendly, target);
+
     foreach(ExObjListSubContainer container,
             p->documentContainer->exObjList->rgChildRec) {
         // Search all ExHyperlinkContainers for specified id
