@@ -236,18 +236,20 @@ public:
     unsigned indentLevel;
     unsigned rotationAngle;
     bool stackedLetters;
+    bool shrinkToFit;
 };
 
 FormatAlignment::FormatAlignment()
 {
     d = new FormatAlignment::Private;
     d->null          = true;
-    d->alignX        = Format::Left;
-    d->alignY        = Format::Middle;
+    d->alignX        = Format::Standard;
+    d->alignY        = Format::Standard;
     d->wrap          = false;
     d->indentLevel   = 0;
     d->rotationAngle = 0;
     d->stackedLetters = false;
+    d->shrinkToFit = false;
 }
 
 // destructor
@@ -279,6 +281,7 @@ FormatAlignment& FormatAlignment::assign(const FormatAlignment& align)
     d->indentLevel   = align.indentLevel();
     d->rotationAngle = align.rotationAngle();
     d->stackedLetters = align.stackedLetters();
+    d->shrinkToFit = align.shrinkToFit();
     return *this;
 }
 
@@ -348,6 +351,17 @@ void FormatAlignment::setStackedLetters(bool b)
     d->null = false;
 }
 
+bool FormatAlignment::shrinkToFit() const
+{
+    return d->shrinkToFit;
+}
+
+void FormatAlignment::setShrinkToFit(bool b)
+{
+    d->shrinkToFit = b;
+    d->null = false;
+}
+
 bool FormatAlignment::operator==(const FormatAlignment& font) const
 {
     return
@@ -356,7 +370,8 @@ bool FormatAlignment::operator==(const FormatAlignment& font) const
         d->wrap == font.d->wrap &&
         d->indentLevel == font.d->indentLevel &&
         d->rotationAngle == font.d->rotationAngle &&
-        d->stackedLetters == font.d->stackedLetters;
+        d->stackedLetters == font.d->stackedLetters &&
+        d->shrinkToFit == font.d->shrinkToFit;
 }
 
 bool FormatAlignment::operator!=(const FormatAlignment& font) const
@@ -367,7 +382,8 @@ bool FormatAlignment::operator!=(const FormatAlignment& font) const
         d->wrap != font.d->wrap ||
         d->indentLevel != font.d->indentLevel ||
         d->rotationAngle != font.d->rotationAngle ||
-        d->stackedLetters != font.d->stackedLetters;
+        d->stackedLetters != font.d->stackedLetters ||
+        d->shrinkToFit != font.d->shrinkToFit;
 }
 
 class FormatBackground::Private
@@ -478,6 +494,8 @@ public:
     Pen rightBorder;
     Pen topBorder;
     Pen bottomBorder;
+    Pen topLeftBorder;
+    Pen bottomLeftBorder;
 };
 
 // constructor
@@ -514,6 +532,8 @@ FormatBorders& FormatBorders::assign(const FormatBorders& border)
     d->rightBorder  = border.rightBorder();
     d->topBorder    = border.topBorder();
     d->bottomBorder = border.bottomBorder();
+    d->topLeftBorder = border.topLeftBorder();
+    d->bottomLeftBorder = border.bottomLeftBorder();
     return *this;
 }
 
@@ -566,13 +586,37 @@ void FormatBorders::setBottomBorder(const Pen& pen)
     d->null = false;
 }
 
+const Pen& FormatBorders::topLeftBorder() const
+{
+    return d->topLeftBorder;
+}
+
+void FormatBorders::setTopLeftBorder(const Pen& pen)
+{
+    d->topLeftBorder = pen;
+    d->null = false;
+}
+
+const Pen& FormatBorders::bottomLeftBorder() const
+{
+    return d->bottomLeftBorder;
+}
+
+void FormatBorders::setBottomLeftBorder(const Pen& pen)
+{
+    d->bottomLeftBorder = pen;
+    d->null = false;
+}
+    
 bool FormatBorders::operator==(const FormatBorders& font) const
 {
     return
         d->leftBorder == font.d->leftBorder &&
         d->rightBorder == font.d->rightBorder &&
         d->topBorder == font.d->topBorder &&
-        d->bottomBorder == font.d->bottomBorder;
+        d->bottomBorder == font.d->bottomBorder &&
+        d->topLeftBorder == font.d->topLeftBorder &&
+        d->bottomLeftBorder == font.d->bottomLeftBorder;
 }
 
 bool FormatBorders::operator!=(const FormatBorders& font) const
@@ -581,7 +625,9 @@ bool FormatBorders::operator!=(const FormatBorders& font) const
         d->leftBorder != font.d->leftBorder ||
         d->rightBorder != font.d->rightBorder ||
         d->topBorder != font.d->topBorder ||
-        d->bottomBorder != font.d->bottomBorder;
+        d->bottomBorder != font.d->bottomBorder ||
+        d->topLeftBorder != font.d->topLeftBorder ||
+        d->bottomLeftBorder != font.d->bottomLeftBorder;
 }
 
 // helper class for Format class
