@@ -1,5 +1,6 @@
 /* Swinder - Portable library for spreadsheet
    Copyright (C) 2003 Ariya Hidayat <ariya@kde.org>
+   Copyright (C) 2010 Carlos Licea <carlos@kdab.com>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -21,9 +22,15 @@
 #include "sheet.h"
 #include "workbook.h"
 #include "ustring.h"
+#include "utils.h"
 
 #include <iostream>
 #include <map>
+#include <QPoint>
+#include <QFont>
+#include <QFontMetricsF>
+#include <QWidget>
+#include <QDebug>
 
 namespace Swinder
 {
@@ -35,11 +42,12 @@ public:
     UString name;
 
     // hash to store cell, FIXME replace with quad-tree
-    std::map<unsigned, Cell*> cells;
+    QHash<unsigned, Cell*> cells;
     unsigned maxRow;
     unsigned maxColumn;
-    std::map<unsigned, Column*> columns;
-    std::map<unsigned, Row*> rows;
+    QHash<unsigned, unsigned> maxCellsInRow;
+    QHash<unsigned, Column*> columns;
+    QHash<unsigned, Row*> rows;
 
     bool visible;
     bool protect;
@@ -61,6 +69,12 @@ public:
     long defaultColWidth;
 
     double zoomLevel;
+    bool showGrid;
+    bool showZeroValues;
+    QPoint firstVisibleCell;
+    bool isPageBreakViewEnabled;
+
+    unsigned long passwd;
 };
 
 }
@@ -71,19 +85,7 @@ Sheet::Sheet(Workbook* wb)
 {
     d = new Sheet::Private();
     d->workbook = wb;
-    d->name = "Sheet"; // FIXME better name ?
-    d->maxRow = 0;
-    d->maxColumn = 0;
-    d->visible      = true;
-    d->protect      = false;
-    d->leftMargin   = 54;  // 0.75 inch
-    d->rightMargin  = 54;  // 0.75 inch
-    d->topMargin    = 72;  // 1 inch
-    d->bottomMargin = 72;  // 1 inch
-    d->autoCalc = true;
-    d->defaultRowHeight = -1;
-    d->defaultColWidth = -1;
-    d->zoomLevel = 1.0; // 100%
+    clear();
 }
 
 Sheet::~Sheet()
@@ -110,19 +112,34 @@ void Sheet::setAutoCalc(bool a)
 void Sheet::clear()
 {
     // delete all cells
-    std::map<unsigned, Cell*>::iterator cell_it;
-    for (cell_it = d->cells.begin(); cell_it != d->cells.end(); ++cell_it)
-        delete cell_it->second;
-
+    qDeleteAll(d->cells);
+    d->cells.clear();
     // delete all columns
-    std::map<unsigned, Column*>::iterator col_it;
-    for (col_it = d->columns.begin(); col_it != d->columns.end(); ++col_it)
-        delete col_it->second;
-
+    qDeleteAll(d->columns);
+    d->columns.clear();
     // delete all rows
-    std::map<unsigned, Row*>::iterator row_it;
-    for (row_it = d->rows.begin(); row_it != d->rows.end(); ++row_it)
-        delete row_it->second;
+    qDeleteAll(d->rows);
+    d->rows.clear();
+    
+    d->name = "Sheet"; // FIXME better name ?
+    d->maxRow = 0;
+    d->maxColumn = 0;
+    d->maxCellsInRow.clear();
+    d->visible      = true;
+    d->protect      = false;
+    d->leftMargin   = 54;  // 0.75 inch
+    d->rightMargin  = 54;  // 0.75 inch
+    d->topMargin    = 72;  // 1 inch
+    d->bottomMargin = 72;  // 1 inch
+    d->autoCalc = true;
+    d->defaultRowHeight = 12;
+    d->defaultColWidth = Column::columnUnitsToPts(8.43 * 256.0);
+    d->zoomLevel = 1.0; // 100%
+    d->showGrid = true;
+    d->showZeroValues = true;
+    d->firstVisibleCell = QPoint(0,0); // A1
+    d->isPageBreakViewEnabled = false;
+    d->passwd = 0; // password protection disabled
 }
 
 UString Sheet::name() const
@@ -137,7 +154,7 @@ void Sheet::setName(const UString& name)
 
 Cell* Sheet::cell(unsigned columnIndex, unsigned rowIndex, bool autoCreate)
 {
-    unsigned hashed = (rowIndex + 1) * 1024 + columnIndex + 1;
+    const unsigned hashed = (rowIndex + 1) * maximalColumnCount + columnIndex + 1;
     Cell* c = d->cells[ hashed ];
 
     // create cell if necessary
@@ -151,6 +168,9 @@ Cell* Sheet::cell(unsigned columnIndex, unsigned rowIndex, bool autoCreate)
 
         if (rowIndex > d->maxRow) d->maxRow = rowIndex;
         if (columnIndex > d->maxColumn) d->maxColumn = columnIndex;
+        
+        if(!d->maxCellsInRow.contains(rowIndex) || columnIndex > d->maxCellsInRow[rowIndex])
+            d->maxCellsInRow[rowIndex] = columnIndex;
     }
 
     return c;
@@ -206,6 +226,13 @@ void Sheet::setMaxColumn(unsigned column)
         d->maxColumn = column;
 }
 
+unsigned Sheet::maxCellsInRow(int rowIndex) const
+{
+    if(d->maxCellsInRow.contains(rowIndex))
+        return d->maxCellsInRow[rowIndex];
+    return 0;
+}
+
 bool Sheet::visible() const
 {
     return d->visible;
@@ -356,6 +383,56 @@ void Sheet::setZoomLevel(double fraction)
     d->zoomLevel = fraction;
 }
 
+bool Sheet::showGrid() const
+{
+    return d->showGrid;
+}
+
+void Sheet::setShowGrid(bool show)
+{
+    d->showGrid = show;
+}
+
+bool Sheet::showZeroValues() const
+{
+    return d->showZeroValues;
+}
+
+void Sheet::setShowZeroValues(bool show)
+{
+    d->showZeroValues = show;
+}
+
+QPoint Sheet::firstVisibleCell() const
+{
+    return d->firstVisibleCell;
+}
+
+void Sheet::setFirstVisibleCell(const QPoint &point)
+{
+    d->firstVisibleCell = point;
+}
+
+bool Sheet::isPageBreakViewEnabled() const
+{
+    return d->isPageBreakViewEnabled;
+}
+
+void Sheet::setPageBreakViewEnabled(bool enabled)
+{
+    d->isPageBreakViewEnabled = enabled;
+}
+    
+unsigned long Sheet::password() const
+{
+    return d->passwd;
+}
+
+void Sheet::setPassword(unsigned long hash)
+{
+    d->passwd = hash;
+}
+
 class Column::Private
 {
 public:
@@ -371,7 +448,7 @@ Column::Column(Sheet* sheet, unsigned index)
     d = new Column::Private;
     d->sheet   = sheet;
     d->index   = index;
-    d->width   = 2560 / 120;
+    d->width   = 0.0;
     d->visible = true;
 }
 
@@ -392,6 +469,9 @@ unsigned Column::index() const
 
 double Column::width() const
 {
+    if( d->width == 0.0)
+        return d->sheet->defaultColWidth();
+
     return d->width;
 }
 
@@ -420,6 +500,37 @@ void Column::setVisible(bool b)
     d->visible = b;
 }
 
+bool Column::operator==(const Column &other) const
+{
+    return width() == other.width() &&
+           visible() == other.visible() &&
+           format() == other.format();
+}
+
+bool Column::operator!=(const Column &other) const
+{
+    return ! (*this == other);
+}
+
+double Column::columnUnitsToPts(const double columnUnits)
+{
+    QFont font("Arial", 10);
+    QFontMetricsF fontMetrics(font);
+    double characterWidth = qMax(fontMetrics.width("0"),qMax(fontMetrics.width("1"),
+    qMax(fontMetrics.width("2"),qMax(fontMetrics.width("3"),
+    qMax(fontMetrics.width("4"),qMax(fontMetrics.width("5"),
+    qMax(fontMetrics.width("6"),qMax(fontMetrics.width("7"),
+    qMax(fontMetrics.width("8"),fontMetrics.width("9"))))))))));
+
+    double width = characterWidth * columnUnits / 256.0; //px
+    width = qRound(width / 8.0 + 0.5) * 8.0;
+
+    QWidget widget;
+    width /= (double)widget.physicalDpiX(); //in
+    width *= 72.0; //pt
+    return width;
+}
+
 class Row::Private
 {
 public:
@@ -435,7 +546,7 @@ Row::Row(Sheet* sheet, unsigned index)
     d = new Row::Private;
     d->sheet   = sheet;
     d->index   = index;
-    d->height  = 10;
+    d->height  = 12;
     d->visible = true;
 }
 
@@ -483,3 +594,15 @@ void Row::setVisible(bool b)
 {
     d->visible = b;
 }
+
+bool Row::operator==(const Row &other) const
+{
+    return height() == other.height() &&
+           visible() == other.visible() &&
+           format() == other.format();
+}
+
+bool Row::operator!=(const Row &other) const
+{
+    return ! (*this == other);
+}
