@@ -23,6 +23,7 @@
  */
 
 #include "XlsxXmlWorksheetReader.h"
+#include "XlsxXmlCommentsReader.h"
 #include "XlsxXmlStylesReader.h"
 #include "XlsxXmlDocumentReader.h"
 #include "XlsxXmlDrawingReader.h"
@@ -76,15 +77,17 @@ XlsxXmlWorksheetReaderContext::XlsxXmlWorksheetReaderContext(
     const QString _path, const QString _file,
     const QMap<QString, MSOOXML::DrawingMLTheme*>& _themes,
     const XlsxSharedStringVector& _sharedStrings,
+    const XlsxComments& _comments,
     const XlsxStyles& _styles,
     MSOOXML::MsooXmlRelationships& _relationships,
     XlsxImport* _import
-    )
+)
         : MSOOXML::MsooXmlReaderContext(&_relationships)
         , worksheetNumber(_worksheetNumber)
         , worksheetName(_worksheetName)
         , themes(&_themes)
         , sharedStrings(&_sharedStrings)
+        , comments(&_comments)
         , styles(&_styles)
         , import(_import)
         , path(_path)
@@ -193,7 +196,7 @@ public:
     int maxRow() const { return m_maxRow; }
     int maxColumn() const { return m_maxColumn; }
     int maxCellsInRow(int rowIndex) const { return m_maxCellsInRow[rowIndex]; }
-    
+
 private:
     QHash<int, Row*> m_rows;
     QHash<int, Column*> m_columns;
@@ -223,6 +226,7 @@ public:
     bool warningAboutWorksheetSizeDisplayed;
     int drawingNumber;
     Sheet* sheet;
+    QHash<int, Cell*> sharedFormulas;
 };
 
 XlsxXmlWorksheetReader::XlsxXmlWorksheetReader(KoOdfWriters *writers)
@@ -305,6 +309,32 @@ void XlsxXmlWorksheetReader::showWarningAboutWorksheetSize()
         "sheet was exceeded.");
 }
 
+inline static QString encodeLabelText(int col, int row)
+{
+    return KSpread::Util::encodeColumnLabelText(col) + QString::number(row);
+}
+
+void XlsxXmlWorksheetReader::saveAnnotation(int col, int row)
+{
+    QString ref(encodeLabelText(col + 1, row + 1));
+    kDebug() << ref;
+    XlsxComment *comment = m_context->comments->value(ref);
+    if (!comment)
+        return;
+    kDebug() << "Saving annotation for cell" << ref;
+    body->startElement("office:annotation");
+    body->startElement("dc:creator");
+    body->addTextNode(comment->author(m_context->comments));
+    body->endElement(); // dc:creator
+    //! @todo support dc:date
+    foreach (const QString& text, comment->texts) {
+        body->startElement("text:p");
+        body->addTextSpan(text);
+        body->endElement(); // text:p
+    }
+    body->endElement(); // office:annotation
+}
+
 #undef CURRENT_EL
 #define CURRENT_EL worksheet
 //! worksheet handler (Worksheet)
@@ -415,7 +445,7 @@ KoFilter::ConversionStatus XlsxXmlWorksheetReader::read_worksheet()
                 body->startElement("table:table-cell");
                 if (Cell* cell = d->sheet->cell(c, r, false)) {
                     const bool hasHyperlink = ! cell->hyperlink.isEmpty();
-                    
+
                     if (!cell->styleName.isEmpty()) {
                         body->addAttribute("table:style-name", cell->styleName);
                     }
@@ -424,7 +454,13 @@ KoFilter::ConversionStatus XlsxXmlWorksheetReader::read_worksheet()
                         body->addAttribute("office:value-type", cell->valueType);
                     }
                     if (!cell->valueAttr.isEmpty()) {
-                        body->addAttribute(cell->valueAttr, cell->valueAttrValue);
+                        // Treat boolean values specially (ODF1.1 chapter 6.7.1)
+                        if (cell->valueAttr == XlsxXmlWorksheetReader::officeBooleanValue)
+                            //! @todo This breaks down if the value is a formula and not constant.
+                            body->addAttribute(cell->valueAttr, 
+                                               cell->valueAttrValue == "0" ? "false" : "true");
+                        else
+                            body->addAttribute(cell->valueAttr, cell->valueAttrValue);
                     }
                     if (!cell->formula.isEmpty()) {
                         body->addAttribute("table:formula", cell->formula);
@@ -435,10 +471,12 @@ KoFilter::ConversionStatus XlsxXmlWorksheetReader::read_worksheet()
                     if (cell->columnsMerged > 1) {
                         body->addAttribute("table:number-columns-spanned", cell->columnsMerged);
                     }
-                    
+
+                    saveAnnotation(c, r);
+
                     if (!cell->text.isEmpty() || !cell->charStyleName.isEmpty() || hasHyperlink) {
                         body->startElement("text:p", false);
-                        if(!cell->charStyleName.isEmpty()) {
+                        if (!cell->charStyleName.isEmpty()) {
                             body->startElement( "text:span" );
                             body->addAttribute( "text:style-name", cell->charStyleName);
                         }
@@ -448,15 +486,15 @@ KoFilter::ConversionStatus XlsxXmlWorksheetReader::read_worksheet()
                             //body->addAttribute("office:target-frame-name", targetFrameName);
                             body->addTextNode(cell->text.isEmpty() ? cell->hyperlink : cell->text);
                             body->endElement(); // text:a
-                        } else if(!cell->text.isEmpty()) {
+                        } else if (!cell->text.isEmpty()) {
                             body->addTextSpan(cell->text);
                         }
-                        if(!cell->charStyleName.isEmpty()) {
+                        if (!cell->charStyleName.isEmpty()) {
                             body->endElement(); // text:span
                         }
                         body->endElement(); // text:p
                     }
-                        
+
                     // handle objects like e.g. charts
                     foreach(XlsxXmlDrawingReaderContext* drawing, cell->drawings) {
                         foreach(XlsxXmlChartReaderContext* chart, drawing->charts) {
@@ -747,7 +785,7 @@ KoFilter::ConversionStatus XlsxXmlWorksheetReader::read_row()
     m_currentColumn = 0;
     Row* row = d->sheet->row(m_currentRow, true);
     row->styleName = processRowStyle(ht);
-    
+
     if (!hidden.isEmpty()) {
         row->hidden = hidden.toInt() > 0;
     }
@@ -779,7 +817,7 @@ static QString convertFormula(const QString& formula)
 {
     if (formula.isEmpty())
         return QString();
-    enum { Start, InArguments, InString, InSheetOrAreaName } state;
+    enum { Start, InArguments, InParenthesizedArgument, InString, InSheetOrAreaName } state;
     state = Start;
     QString result = '=' + formula;
     for(int i = 1; i < result.length(); ++i) {
@@ -796,6 +834,14 @@ static QString convertFormula(const QString& formula)
                 state = InSheetOrAreaName;
             else if (ch == ',')
                 result[i] = ';'; // replace argument delimiter
+            else if (ch == '(')
+                state = InParenthesizedArgument;
+            break;
+        case InParenthesizedArgument:
+            if (ch == ',')
+                result[i] = '~'; // union operator
+            else if (ch == ')')
+                state = InArguments;
             break;
         case InString:
             if (ch == '"')
@@ -856,9 +902,11 @@ static QString convertFormulaReference(Cell* referencedCell, Cell* thisCell)
                 const QString ref = result.mid(cellReferenceStart, i - cellReferenceStart);
                 QRegExp rx("(|\\$)[A-Za-z]+[0-9]+");
                 if (rx.exactMatch(ref)) {
-                    const int c = KSpread::Util::decodeColumnLabelText(ref) + thisCell->column - referencedCell->column;
+                    const int c = KSpread::Util::decodeColumnLabelText(ref)
+                                  + thisCell->column - referencedCell->column;
                     const int r = KSpread::Util::decodeRowLabelText(ref) + thisCell->row - referencedCell->row;
-                    result = result.replace(cellReferenceStart, i - cellReferenceStart, KSpread::Util::encodeColumnLabelText(c) + QString::number(r));
+                    result = result.replace(cellReferenceStart,
+                                            i - cellReferenceStart, encodeLabelText(c, r));
                     state = InArguments;
                 }
             }
@@ -923,7 +971,7 @@ KoFilter::ConversionStatus XlsxXmlWorksheetReader::read_c()
     kDebug() << "styleId:" << styleId;
     const XlsxCellFormat* cellFormat = m_context->styles->cellFormat(styleId);
     const QString numberFormat = cellFormat->applyNumberFormat ? m_context->styles->numberFormatString( cellFormat->numFmtId ) : QString();
-    
+
     const QString formattedStyle = d->processValueFormat( numberFormat );
     QString charStyleName;
 
@@ -943,7 +991,7 @@ KoFilter::ConversionStatus XlsxXmlWorksheetReader::read_c()
         if( !charStyleName.isEmpty() ) {
             cell->charStyleName = charStyleName;
         }
- 
+
         /* depending on type: 18.18.11 ST_CellType (Cell Type), p. 2679:
             b (Boolean) Cell containing a boolean.
             d (Date) Cell contains a date in the ISO 8601 format.
@@ -958,7 +1006,7 @@ KoFilter::ConversionStatus XlsxXmlWorksheetReader::read_c()
 
             Converting into values described in ODF1.1: "6.7.1. Variable Value Types and Values".
         */
-       
+
         if (t == QLatin1String("s")) {
             bool ok;
             const int stringIndex = m_value.toInt(&ok);
@@ -1093,35 +1141,15 @@ KoFilter::ConversionStatus XlsxXmlWorksheetReader::read_f()
     // Range of cells which the formula applies to. Only required for shared formula, array
     // formula or data table. Only written on the master formula, not subsequent formula's
     // belonging to the same shared group, array, or data table.
-    TRY_READ_ATTR(ref)
-    
+    //TRY_READ_ATTR(ref)
     // Type of formula. The possible values defined by the ST_CellFormulaType (§18.18.6), p. 2677
-    TRY_READ_ATTR(t)    
-    if (!t.isEmpty()) {
-        if (t == QLatin1String("shared")) {
-            /* Shared Group Index, p. 1815
-            Optional attribute to optimize load performance by sharing formulas.
-            When a formula is a shared formula (t value is shared) then this value indicates the
-            group to which this particular cell's formula belongs. The first formula in a group of
-            shared formulas is saved in the f element. This is considered the 'master' formula cell.
-            Subsequent cells sharing this formula need not have the formula written in their f
-            element. Instead, the attribute si value for a particular cell is used to figure what the
-            formula expression should be based on the cell's relative location to the master formula
-            cell.
-            */
-            //TRY_READ_ATTR(si)
-            //int sharedGroupIndex;
-            //STRING_TO_INT(si, sharedGroupIndex, "f@si")
-        }
-        else if (t == QLatin1String("normal")) { // Formula is a regular cell formula
-            
-        }
-        else if (t == QLatin1String("array")) { // Formula is an array formula
-            //! @todo array
-        }
-        else if (t == QLatin1String("dataTable")) { // Formula is a data table formula
-            //! @todo dataTable
-        }
+    TRY_READ_ATTR(t)
+
+    // Shared formula groups.
+    int sharedGroupIndex = -1;
+    if (t == QLatin1String("shared")) {
+        TRY_READ_ATTR(si)
+        STRING_TO_INT(si, sharedGroupIndex, "f@si")
     }
 
     while (!atEnd() && !hasError()) {
@@ -1131,7 +1159,31 @@ KoFilter::ConversionStatus XlsxXmlWorksheetReader::read_f()
             cell->formula = convertFormula(text().toString());
         }
     }
+    
+    if (!t.isEmpty()) {
+        if (t == QLatin1String("shared")) {
+            if (sharedGroupIndex >= 0) {
+                /* Shared Group Index, p. 1815
+                Optional attribute to optimize load performance by sharing formulas.
+                When a formula is a shared formula (t value is shared) then this value indicates the
+                group to which this particular cell's formula belongs. The first formula in a group of
+                shared formulas is saved in the f element. This is considered the 'master' formula cell.
+                Subsequent cells sharing this formula need not have the formula written in their f
+                element. Instead, the attribute si value for a particular cell is used to figure what the
+                formula expression should be based on the cell's relative location to the master formula
+                cell.
+                */
+                if (d->sharedFormulas.contains(sharedGroupIndex)) {
+                    if (cell->formula.isEmpty()) // don't do anything if the cell already defines a formula
+                        cell->formula = convertFormulaReference(d->sharedFormulas[sharedGroupIndex], cell);
+                } else if (!cell->formula.isEmpty()) { // is this cell the master cell?
+                    d->sharedFormulas[sharedGroupIndex] = cell;
+                }
+            }
+        }
+    }
 
+    /*
     if (!ref.isEmpty()) {
         const int pos = ref.indexOf(':');
         if (pos > 0) {
@@ -1145,14 +1197,15 @@ KoFilter::ConversionStatus XlsxXmlWorksheetReader::read_f()
                 for (int col = c1; col <= c2; ++col) {
                     for (int row = r1; row <= r2; ++row) {
                         if (col != m_currentColumn || row != m_currentRow) {
-                            Cell* c = d->sheet->cell(col, row, true);
-                            c->formula = convertFormulaReference(cell, c);
+                            if (Cell* c = d->sheet->cell(col, row, true))
+                                c->formula = convertFormulaReference(cell, c);
                         }
                     }
                 }
             }
         }
     }
+    */
 
     READ_EPILOGUE
 }
@@ -1286,7 +1339,7 @@ KoFilter::ConversionStatus MSOOXML_CURRENT_CLASS::read_drawing()
             delete context;
             return result;
         }
-        
+
         if (context->m_positions.contains(XlsxXmlDrawingReaderContext::FromAnchor)) {
             XlsxXmlDrawingReaderContext::Position pos = context->m_positions[XlsxXmlDrawingReaderContext::FromAnchor];
             Cell* cell = d->sheet->cell(pos.m_col, pos.m_row, true);
@@ -1317,7 +1370,7 @@ KoFilter::ConversionStatus XlsxXmlWorksheetReader::read_hyperlink()
             // it follows a hack to get right of the prepended m_context->path...
             if (link.startsWith(m_context->path))
                 link = link.mid(m_context->path.length()+1);
-                
+
             Cell* cell = d->sheet->cell(col, row, true);
             cell->hyperlink = link;
         }
