@@ -24,11 +24,15 @@
 #ifndef DOCXXMLDOCREADER_H
 #define DOCXXMLDOCREADER_H
 
+#include <QBuffer>
+#include <QString>
+
 #include <MsooXmlCommonReader.h>
 #include <MsooXmlThemesReader.h>
 #include "DocxXmlCommentsReader.h"
 #include "DocxXmlNotesReader.h"
 
+#include <KoXmlWriter.h>
 #include <KoGenStyle.h>
 #include <styles/KoCharacterStyle.h>
 
@@ -68,11 +72,16 @@ protected:
     KoFilter::ConversionStatus read_rPr(rPrCaller caller);
     KoFilter::ConversionStatus read_pPr();
     KoFilter::ConversionStatus read_sectPr();
+    KoFilter::ConversionStatus read_footerReference();
+    KoFilter::ConversionStatus read_headerReference();
     KoFilter::ConversionStatus read_pgSz();
+    KoFilter::ConversionStatus read_textDirection();
     KoFilter::ConversionStatus read_pgMar();
     KoFilter::ConversionStatus read_pgBorders();
     KoFilter::ConversionStatus read_object();
     KoFilter::ConversionStatus read_ind();
+    KoFilter::ConversionStatus read_outline();
+    KoFilter::ConversionStatus read_framePr();
     KoFilter::ConversionStatus read_OLEObject();
     //KoFilter::ConversionStatus read_commentRangeEnd();
     KoFilter::ConversionStatus read_commentRangeStart();
@@ -80,6 +89,9 @@ protected:
     KoFilter::ConversionStatus read_footnoteReference();
     KoFilter::ConversionStatus read_hyperlink();
     KoFilter::ConversionStatus read_drawing();
+    KoFilter::ConversionStatus read_ptab();
+    KoFilter::ConversionStatus read_tabs();
+    KoFilter::ConversionStatus read_tab();
     KoFilter::ConversionStatus read_i();
     KoFilter::ConversionStatus read_b();
     KoFilter::ConversionStatus read_u();
@@ -93,6 +105,8 @@ protected:
     KoFilter::ConversionStatus read_shd(shdCaller caller);
     KoFilter::ConversionStatus read_rFonts();
     KoFilter::ConversionStatus read_pStyle();
+    KoFilter::ConversionStatus read_rStyle();
+    KoFilter::ConversionStatus read_fldSimple();
     KoFilter::ConversionStatus read_lastRenderedPageBreak();
     KoFilter::ConversionStatus read_instrText();
     KoFilter::ConversionStatus read_fldChar();
@@ -100,20 +114,54 @@ protected:
     KoFilter::ConversionStatus read_dstrike();
     KoFilter::ConversionStatus read_caps();
     KoFilter::ConversionStatus read_smallCaps();
+    KoFilter::ConversionStatus read_w();
     KoFilter::ConversionStatus read_color();
     KoFilter::ConversionStatus read_highlight();
     KoFilter::ConversionStatus read_vertAlign();
     KoFilter::ConversionStatus read_lang();
     KoFilter::ConversionStatus read_background();
     KoFilter::ConversionStatus read_pBdr();
+    KoFilter::ConversionStatus read_tbl();
+    KoFilter::ConversionStatus read_tblPr();
+    KoFilter::ConversionStatus read_tblGrid();
+    KoFilter::ConversionStatus read_gridCol();
+    KoFilter::ConversionStatus read_tr();
+    KoFilter::ConversionStatus read_tc();
+    KoFilter::ConversionStatus read_tcPr();
+
+    enum posOffsetCaller {
+        posOffset_positionH,
+        posOffset_positionV
+    };
+
+    enum alignCaller {
+        align_positionH,
+        align_positionV
+    };
+    KoFilter::ConversionStatus read_align(alignCaller caller);
+
+    KoFilter::ConversionStatus read_inline();
+    KoFilter::ConversionStatus read_docPr();
+    KoFilter::ConversionStatus read_anchor();
+    KoFilter::ConversionStatus read_positionH();
+    KoFilter::ConversionStatus read_positionV();
+    KoFilter::ConversionStatus read_posOffset(posOffsetCaller caller);
+    KoFilter::ConversionStatus read_wrapSquare();
+    KoFilter::ConversionStatus read_wrapTight();
+    KoFilter::ConversionStatus read_wrapThrough();
 
     KoGenStyle m_currentPageStyle;
+    KoGenStyle m_masterPageStyle;
 
     DocxXmlDocumentReaderContext* m_context;
 
 private:
     void init();
 
+    //! Writes text:p/draw:rect elements.
+    //! Uses m_currentObjectWidthCm and m_currentObjectHeightCm for size (defaults to 2cm/2cm)
+    void writeRect();
+
     enum BorderSide {
         TopBorder, BottomBorder, LeftBorder, RightBorder
     };
@@ -153,11 +201,47 @@ private:
     QString m_complexCharValue;
 
     enum ComplexCharStatus {
-       NoneAllowed, InstrAllowed, InstrExecute
+        NoneAllowed, InstrAllowed, InstrExecute
     };
     //! State of fldChar
     ComplexCharStatus m_complexCharStatus;
 
+    enum DropCapStatus {
+        NoDropCap, DropCapRead, DropCapDone
+    };
+    //! State of dropCap
+    DropCapStatus m_dropCapStatus;
+
+    //! Buffer where first letters of drop cap are read
+    QBuffer m_dropCapBuffer;
+    KoXmlWriter* m_dropCapWriter;
+    QString m_dropCapLines;
+    qreal   m_dropCapDistance;
+
+    uint m_currentTableNumber; //!< table counter, from 0
+    uint m_currentTableRowNumber; //!< row counter, from 0, initialized in read_tbl()
+    uint m_currentTableColumnNumber; //!< column counter, from 0, initialized in read_tr()
+    KoGenStyle m_currentTableStyle;
+    KoGenStyle m_currentTableRowStyle;
+    KoGenStyle m_currentTableCellStyle;
+    QString m_currentTableName;
+    qreal m_currentTableWidth; //!< in cm
+
+    /*! true if w:object/v:shape or w:object/o:OLEObject has been handled, .
+     When w:object/o:OLEObject is visited and m_objectRectInitialized is true, handling
+     w:object/o:OLEObject is (except for copying the OLE binary) skipped because
+     w:object/v:shape is of higher priority.
+     This flag is reset to false each time read_object() is called. */
+    bool m_objectRectInitialized;
+
+    //!< Width of the object. Set in read_OLEObject() or read_shape(). Used in writeRect().
+    //! If both w:object/v:shape and w:object/o:OLEObject exist, information from v:shape is used.
+    QString m_currentObjectWidthCm;
+
+    QString m_currentObjectHeightCm; //!< See m_currentObjectWidthCm for description
+    QString m_currentObjectXCm; //!< See m_currentObjectWidthCm for description
+    QString m_currentObjectYCm; //!< See m_currentObjectWidthCm for description
+
 #include <MsooXmlCommonReaderMethods.h>
 #include <MsooXmlCommonReaderDrawingMLMethods.h>
 
