@@ -69,12 +69,15 @@ KWordTextHandler::KWordTextHandler(wvWare::SharedPtr<wvWare::Parser> parser, KoX
     , m_index(0)
     , m_currentTable(0)
     , m_paragraph(0)
+    , m_hasStoredDropCap(false)
     , m_insideField(false)
     , m_fieldAfterSeparator(false)
     , m_fieldType(0)
     , m_insideFootnote(false)
     , m_footnoteWriter(0)
     , m_footnoteBuffer(0)
+    , m_insideBookmark(false)
+    , m_bookmarkWriter(0)
     , m_insideAnnotation(false)
     , m_annotationWriter(0)
     , m_annotationBuffer(0)
@@ -181,6 +184,39 @@ void KWordTextHandler::sectionStart(wvWare::SharedPtr<const wvWare::Word97::SEP>
         m_bodyWriter->addAttribute("text:name", sectionName);
         m_bodyWriter->addAttribute("text:style-name", sectionStyleName);
     }
+
+    if (sep->nLnnMod != 0) {    // if line numbering modulus is not 0, do line numbering
+        if (m_mainStyles) {
+            QString lineNumbersStyleName = QString("Standard");     // set default line numbers style
+
+            if (m_document) {                                       // if we have m_document
+                QString lineNumbersStyleNameTemp = m_document->lineNumbersStyleName();
+
+                if (lineNumbersStyleNameTemp.isEmpty() == false) {      // if the style name is not empty
+                    lineNumbersStyleName = lineNumbersStyleNameTemp;    // use it
+                }
+            }
+
+            // prepare string for line numbering configuration
+            QString lineNumberingConfig("<text:linenumbering-configuration text:style-name=\"%1\" "
+                                        "style:num-format=\"1\" text:number-position=\"left\" text:increment=\"1\"/>");
+
+            m_mainStyles->insertRawOdfStyles(KoGenStyles::DocumentStyles,
+                                             lineNumberingConfig.arg(lineNumbersStyleName).toLatin1());
+
+            KoGenStyle *normalStyle = m_mainStyles->styleForModification(QString("Normal"));
+
+            // if got Normal style, insert line numbering configuration in it
+            if (normalStyle) {
+                normalStyle->addProperty(QString("text:number-lines"), QString("true"));
+                normalStyle->addProperty(QString("text:line-number"), QString("0"));
+            } else {
+                kWarning() << "Could not find Normal style, numbering not added!";
+            }
+
+        }
+    }
+
 }
 
 void KWordTextHandler::sectionEnd()
@@ -336,6 +372,66 @@ void KWordTextHandler::footnoteFound(wvWare::FootnoteData::Type type,
     //varElem.appendChild( footnoteElem );
 }
 
+void KWordTextHandler::bookmarkFound( wvWare::UString characters, wvWare::UString name, wvWare::SharedPtr<const wvWare::Word97::CHP> chp,
+                                      const wvWare::BookmarkFunctor& parseBookmark)
+{
+    // Get the text between the bookmark-start and the bookmark-end
+    QString bookmarkText;
+    int textIndex = 0;
+    int length = characters.length();
+    while (textIndex != length) {
+        bookmarkText.append(characters[textIndex].unicode());
+        ++textIndex;
+    }
+
+    // Get the name of the bookmark
+    QString bookmarkName;
+    int nameIndex = 0;
+    int nameLength = name.length();
+    while (nameIndex != nameLength) {
+        bookmarkName.append(name[nameIndex].unicode());
+        ++nameIndex;
+    }
+
+    m_insideBookmark = true;
+
+    //create temp writer for bookmark content that we'll add to m_paragraph
+    m_bookmarkBuffer = new QBuffer();
+    m_bookmarkBuffer->open(QIODevice::WriteOnly);
+    m_bookmarkWriter = new KoXmlWriter(m_bookmarkBuffer);
+
+    // bookmark-start
+    m_bookmarkWriter->startElement("text:bookmark-start");
+    m_bookmarkWriter->addAttribute("text:name", bookmarkName);
+    m_bookmarkWriter->endElement();
+
+    // Text
+    // If we are in an hyperlink, we should not add the text because it will be out of control after that.
+    if (m_hyperLinkActive) {
+        m_hyperLinkList.last().append(bookmarkText);
+    } else {
+        m_bookmarkWriter->addTextNode(bookmarkText);
+    }
+
+    // bookmark-end
+    m_bookmarkWriter->startElement("text:bookmark-end");
+    m_bookmarkWriter->addAttribute("text:name", bookmarkName);
+    m_bookmarkWriter->endElement();
+
+    emit bookmarkFound(new wvWare::BookmarkFunctor(parseBookmark));
+
+    m_insideBookmark = false;
+
+    QString contents = QString::fromUtf8(m_bookmarkBuffer->buffer(), m_bookmarkBuffer->buffer().size());
+    m_paragraph->addRunOfText(contents, 0, QString(""), m_parser->styleSheet());
+
+    //cleanup
+    delete m_bookmarkWriter;
+    m_bookmarkWriter = 0;
+    delete m_bookmarkBuffer;
+    m_bookmarkBuffer = 0;
+}
+
 
 void KWordTextHandler::annotationFound( wvWare::UString characters, wvWare::SharedPtr<const wvWare::Word97::CHP> chp,
                                         const wvWare::AnnotationFunctor& parseAnnotation)
@@ -379,7 +475,6 @@ void KWordTextHandler::annotationFound( wvWare::UString characters, wvWare::Shar
     m_annotationBuffer = 0;
 }
 
-
 //create an element for the variable
 QDomElement KWordTextHandler::insertVariable(int type, wvWare::SharedPtr<const wvWare::Word97::CHP> chp, const QString& format)
 {
@@ -412,6 +507,7 @@ void KWordTextHandler::tableRowFound(const wvWare::TableRowFunctor& functor, wvW
         static int s_tableNumber = 0;
         m_currentTable = new KWord::Table();
         m_currentTable->name = i18n("Table %1", ++s_tableNumber);
+        m_currentTable->tap = tap;
         //insertAnchor( m_currentTable->name );
     }
 
@@ -426,7 +522,7 @@ void KWordTextHandler::tableRowFound(const wvWare::TableRowFunctor& functor, wvW
 #ifdef IMAGE_IMPORT
 void KWordTextHandler::pictureFound(const wvWare::PictureFunctor& pictureFunctor,
                                     wvWare::SharedPtr<const wvWare::Word97::PICF> picf,
-                                    wvWare::SharedPtr<const wvWare::Word97::CHP> /*chp*/)
+                                    wvWare::SharedPtr<const wvWare::Word97::CHP> chp)
 {
     kDebug(30513);
 
@@ -466,7 +562,7 @@ void KWordTextHandler::pictureFound(const wvWare::PictureFunctor& pictureFunctor
         emit pictureFound(frameName, pictureName, &writer, new wvWare::PictureFunctor(pictureFunctor));
         //now add content to our current paragraph
         QString contents = QString::fromUtf8(buf.buffer(), buf.buffer().size());
-        m_paragraph->addRunOfText(contents, 0, QString(""), m_parser->styleSheet());
+        m_paragraph->addRunOfText(contents, chp, QString(""), m_parser->styleSheet(), true);
 
     } else {
         //signal picture without a writer, because it can just write to the default (bodyWriter)
@@ -548,6 +644,8 @@ void KWordTextHandler::paragraphStart(wvWare::SharedPtr<const wvWare::ParagraphP
     bool inStylesDotXml = false;
     if (m_insideFootnote) {
         writer = m_footnoteWriter;
+    } else if (m_insideBookmark) {
+        writer = m_bookmarkWriter;
     } else if (m_insideDrawing) {
         writer = m_drawingWriter;
     } else if (document()->writingHeader()) {
@@ -615,12 +713,13 @@ void KWordTextHandler::paragraphStart(wvWare::SharedPtr<const wvWare::ParagraphP
     // Now that the bookkeeping is taken care of for old paragraphs,
     // then actually create the new one.
     kDebug(30513) << "create new Paragraph";
-    m_paragraph = new Paragraph(m_mainStyles, inStylesDotXml, isHeading, outlineLevel);
+    m_paragraph = new Paragraph(m_mainStyles, inStylesDotXml, isHeading, m_document->writingHeader(), outlineLevel);
 
     //if ( m_bInParagraph )
     //    paragraphEnd();
     //m_bInParagraph = true;
     //kDebug(30513) <<"paragraphStart. style index:" << paragraphProperties->pap().istd;
+
     kDebug(30513) << "set paragraph properties";
     m_paragraph->setParagraphProperties(paragraphProperties);
     const wvWare::StyleSheet& styles = m_parser->styleSheet();
@@ -640,17 +739,24 @@ void KWordTextHandler::paragraphStart(wvWare::SharedPtr<const wvWare::ParagraphP
         kWarning() << "paragraphProperties was NOT set";
     }
 
-    //check to see if we need a master page name attribute
-    if (document()->writeMasterStyleName() && !document()->writingHeader()) {
+    //check if we need a master page name attribute
+    if (document()->writeMasterPageName() && !document()->writingHeader()) {
         m_paragraph->getOdfParagraphStyle()->addAttribute("style:master-page-name",
-							  document()->masterStyleName());
-        document()->set_writeMasterStyleName(false);
+							  document()->masterPageName());
+        document()->set_writeMasterPageName(false);
     }
 } //end paragraphStart()
 
 void KWordTextHandler::paragraphEnd()
 {
-    kDebug(30513);
+    kDebug(30513) << "-----------------------------------------------";
+
+    // If the last paragraph was a drop cap paragraph, combine it with
+    // this one.
+    if (m_hasStoredDropCap) {
+        kDebug(30513) << "combine paragraphs for drop cap" << m_dropCapString;
+        m_paragraph->addDropCap(m_dropCapString, m_dcs_fdct, m_dcs_lines, m_dropCapDistance);
+    }
 
     //clear our paragraph flag
     //m_bInParagraph = false;
@@ -661,6 +767,9 @@ void KWordTextHandler::paragraphEnd()
     if (m_insideFootnote) {
         kDebug(30513) << "writing a footnote";
         m_paragraph->writeToFile(m_footnoteWriter);
+    } else if (m_insideBookmark) {
+        kDebug(30513) << "writing an bookmark";
+        m_paragraph->writeToFile(m_bookmarkWriter);
     } else if (m_insideAnnotation) {
         kDebug(30513) << "writing an annotation";
         m_paragraph->writeToFile(m_annotationWriter);
@@ -669,8 +778,26 @@ void KWordTextHandler::paragraphEnd()
         m_paragraph->writeToFile(m_drawingWriter);
     } else if (!document()->writingHeader()) {
         kDebug(30513) << "writing to body";
+
+        // Need to call this before the drop cap test just below,
+        // because here is where the style is applied.
         m_paragraph->writeToFile(m_bodyWriter);
 
+        // If this paragraph is a drop cap, it should be combined with
+        // the next paragraph.  So store the drop cap data for future
+        // use.  However, only do this if the last paragraph was *not*
+        // a dropcap.
+        if (!m_hasStoredDropCap && m_paragraph->dropCapStatus() == Paragraph::IsDropCapPara) {
+            m_paragraph->getDropCapData(&m_dropCapString, &m_dcs_fdct, &m_dcs_lines,
+                                        &m_dropCapDistance);
+            m_hasStoredDropCap = true;
+            kDebug(30513) << "saving drop cap data in texthandler" << m_dropCapString;
+        }
+        else {
+            // Remove the traces of the drop cap for the next round.
+            m_hasStoredDropCap = false;
+            m_dropCapString.clear();
+        }
     } else {
         kDebug(30513) << "writing a header";
         m_paragraph->writeToFile(document()->headerWriter());
@@ -678,7 +805,6 @@ void KWordTextHandler::paragraphEnd()
 
     delete m_paragraph;
     m_paragraph = 0;
-
 }//end paragraphEnd()
 
 void KWordTextHandler::fieldStart(const wvWare::FLD* fld, wvWare::SharedPtr<const wvWare::Word97::CHP> /*chp*/)
@@ -688,21 +814,26 @@ void KWordTextHandler::fieldStart(const wvWare::FLD* fld, wvWare::SharedPtr<cons
     m_insideField = true;
     m_fieldAfterSeparator = false;
     m_fieldValue = "";
+    m_RunOfTextCollect = false;     // do not collect strings in runOfText (yet)
+    m_RunOfTextStrings = "";
 
     //check to see if we can process this field type or not
     switch (m_fieldType) {
     case 26:
     case 33:
-    case 37: // TAB
+        m_paragraph->setContainsPageNumberField(true);
+        break;
+    case 37: // Pageref
+        kDebug(30513) << "processing field... Pageref";
         if (m_hyperLinkActive) {
             m_fieldType = 88;
-        } else {
-            kDebug(30513) << "can't process field, just outputting text into document...";
-            m_fieldType = -1; //set m_fieldType for unprocessed field
         }
         break;
+    case 49:
+        m_RunOfTextCollect = true;     // do collect strings in runOfText
+        break;
     case 88:  // HyperLink
-        kDebug(30513) << "processing field...";
+        kDebug(30513) << "processing field... HyperLink";
         m_hyperLinkActive = true;
         break;
     default:
@@ -719,11 +850,13 @@ void KWordTextHandler::fieldSeparator(const wvWare::FLD* /*fld*/, wvWare::Shared
 
 void KWordTextHandler::fieldEnd(const wvWare::FLD* /*fld*/, wvWare::SharedPtr<const wvWare::Word97::CHP> chp)
 {
-    Q_UNUSED(chp);
+//    Q_UNUSED(chp);
 
     kDebug(30513);
     //process different fields
     //we could be writing to content or styles.xml (in a header)
+    
+    m_RunOfTextCollect = false;     // do not collect strings anymore
 
     //create temp writer that we'll add to m_paragraph
     QBuffer buf;
@@ -739,12 +872,52 @@ void KWordTextHandler::fieldEnd(const wvWare::FLD* /*fld*/, wvWare::SharedPtr<co
         writer.addAttribute("text:select-page", "current");
         writer.endElement();
         break;
+    case 37:  // Pageref
+        if (m_bookmarkRef.length() == 2) {
+            // if we have a pageref, two part parts are necessary:
+            // - the first on in which is described the reference
+            // - the second in which is written the text to print
+            if (m_bookmarkRef[0].contains("PAGEREF")) {
+                // we need to keep only the reference
+                m_bookmarkRef[0].remove("PAGEREF");
+                int pos = m_bookmarkRef[0].indexOf('\\');
+                while (pos != -1)
+                {
+                    m_bookmarkRef[0].replace(pos, 2, "");
+                    pos = m_bookmarkRef[0].indexOf('\\');
+                }
+                m_bookmarkRef[0].remove(' ');
+                m_bookmarkRef[1].remove(' ');
+                // Now, we have everything we needed, let's write a bookmark-ref
+                writer.startElement("text:bookmark-ref");
+                writer.addAttribute("text:reference-format","page");
+                writer.addAttribute("text:ref-name",m_bookmarkRef[0]);
+                writer.addTextNode(m_bookmarkRef[1]);
+                writer.endElement();
+            }
+        }
+        break;
+    case 49:    // combined characters stored as 'equation'
+        {
+            QRegExp rx("eq \\\\o\\(\\\\s\\\\up 36\\(([^\\)]*)\\),\\\\s\\\\do 12\\(([^\\)]*)\\)\\)");
+            int where = rx.indexIn(m_RunOfTextStrings);
+
+            if (where != -1) {
+                QString combinedCharacters = rx.cap(1) + rx.cap(2);
+                if (!combinedCharacters.isEmpty()) {
+                    m_paragraph->setCombinedCharacters(true);
+                    m_paragraph->addRunOfText(combinedCharacters, chp, QString(""), m_parser->styleSheet());
+                    m_paragraph->setCombinedCharacters(false);
+                }
+            }
+        }
+        break;
     case 88:  // HyperLink
         QList<QString> fullList(m_hyperLinkList);
         QList<QString> emptyList;
         QString hyperlink;
-
         for (int i = 0; i < fullList.size(); i++) {
+            fullList[i].replace("\\l","");
             if (fullList[i].contains("PAGEREF")) {
                 fullList.removeAt(i);
             }
@@ -821,17 +994,31 @@ void KWordTextHandler::runOfText(const wvWare::UString& text, wvWare::SharedPtr<
     QString newText(Conversion::string(text));
     kDebug(30513) << newText;
 
+    m_RunOfTextStrings += newText;
+
     //we don't want to do anything with an empty string
     if (newText.isEmpty())
         return;
 
     // This method is called twice for each hyperlink. Save link
     // data to m_hyperLinkList to handle it later in fieldEnd -method.
-    if (m_insideField && m_fieldType == 88) {
+   if (m_insideField && m_fieldType == 88) {
         m_hyperLinkList.append(newText);
         return;
     }
 
+    // This method is called twice for each pageref (bookmark-ref). Save link
+    // data to m_bookmarkRef to handle it later in fieldEnd -method.
+    if (m_insideField && m_fieldType == 37) {
+        if (m_hyperLinkActive) {
+            m_hyperLinkList.append(newText);
+            m_fieldType = 88;
+        } else {
+            m_bookmarkRef.append(newText);
+        }
+        return;
+    }
+
     // text after fieldStart and before fieldSeparator is useless
     if (m_insideField && !m_fieldAfterSeparator) {
         kDebug(30513) << "Ignoring this text in first part of field.";
@@ -842,7 +1029,6 @@ void KWordTextHandler::runOfText(const wvWare::UString& text, wvWare::SharedPtr<
     if (m_insideField && m_fieldAfterSeparator && (m_fieldType > 0)) {
         kDebug(30513) << "adding this text to field value.";
         m_fieldValue.append(newText);
-        m_fieldType = 0;
         return;
     }
 
