@@ -78,6 +78,9 @@ public:
     virtual void footnoteFound(wvWare::FootnoteData::Type type, wvWare::UString characters,
                                wvWare::SharedPtr<const wvWare::Word97::CHP> chp,
                                const wvWare::FootnoteFunctor& parseFootnote);
+    virtual void bookmarkFound(wvWare::UString characters, wvWare::UString name,
+                               wvWare::SharedPtr<const wvWare::Word97::CHP> chp,
+                               const wvWare::BookmarkFunctor& parseBookmark);
     virtual void annotationFound(wvWare::UString characters,
                                  wvWare::SharedPtr<const wvWare::Word97::CHP> chp,
                                  const wvWare::AnnotationFunctor& parseAnnotation);
@@ -104,6 +107,9 @@ public:
     void setDocument(Document * document) {
         m_document = document;
     }
+    bool pageNumberFieldType() const {
+        return m_fieldType == 33;
+    }
 
     // Write a <FORMAT> tag from the given CHP
     // Returns that element into pChildElement if set (in that case even an empty FORMAT can be appended)
@@ -125,6 +131,7 @@ signals:
     void sectionEnd(wvWare::SharedPtr<const wvWare::Word97::SEP>);
     void subDocFound(const wvWare::FunctorBase* parsingFunctor, int data);
     void footnoteFound(const wvWare::FunctorBase* parsingFunctor, int data);
+    void bookmarkFound(const wvWare::FunctorBase* parsingFunctor);
     void annotationFound(const wvWare::FunctorBase* parsingFunctor, int data);
     void headersFound(const wvWare::FunctorBase* parsingFunctor, int data);
     void tableFound(KWord::Table* table);
@@ -183,7 +190,15 @@ private:
 
     KWord::Table* m_currentTable;
     //pointer to paragraph object
-    Paragraph* m_paragraph;
+    Paragraph *m_paragraph;
+#if 1
+    bool       m_hasStoredDropCap; // True if the previous paragraph was a dropcap 
+    int        m_dcs_fdct;
+    int        m_dcs_lines;
+    qreal      m_dropCapDistance;
+#endif
+    QString    m_dropCapString;
+    //Paragraph *m_dropCapParagraph; // These are stored separately by MS Office.
 
     QString m_fieldValue;
     bool m_insideField;
@@ -195,10 +210,17 @@ private:
     KoXmlWriter* m_footnoteWriter; //write the footnote data, then add it to bodyWriter
     QBuffer* m_footnoteBuffer; //buffer for the footnote data
 
+    bool m_insideBookmark;
+    KoXmlWriter* m_bookmarkWriter; //add the bookmark to bodyWriter
+    QBuffer* m_bookmarkBuffer; //buffer for the bookmark data
+
     bool m_insideAnnotation;
     KoXmlWriter* m_annotationWriter; //write the annotation data, then add it to bodyWriter
     QBuffer* m_annotationBuffer; //buffer for the annotation data
 
+    bool    m_RunOfTextCollect;         // when this is enabled, the m_RunOfTextStrings collects strings when runOfText is called
+    QString m_RunOfTextStrings;         // collection of strings passed to runOfText
+
     bool m_insideDrawing;
     KoXmlWriter* m_drawingWriter; //write the drawing data, then add it to bodyWriter
 
@@ -212,6 +234,7 @@ private:
 
     QList<QString> m_hyperLinkList;
     bool m_hyperLinkActive;
+    QList<QString> m_bookmarkRef;
 };
 
 #endif // TEXTHANDLER_H
