@@ -38,6 +38,7 @@ namespace wvWare {
     typedef Functor<Parser9x, TableRowData> TableRowFunctor;
     typedef Functor<Parser9x, FootnoteData> FootnoteFunctor;
     typedef Functor<Parser9x, PictureData> PictureFunctor;
+    typedef Functor<Parser9x, BookmarkData> BookmarkFunctor;
 
     /**
      * This class allows to replace the character values of some
@@ -98,6 +99,16 @@ namespace wvWare {
          */
         virtual void footnoteEnd();
 
+        /**
+        * Every time you invoke a @ref BookmarkFunctor this method will be called.
+        * Note that @ref BookmarkFunctor is also emitted when we find endnotes.
+        */
+        virtual void bookmarkStart();
+        /**
+        * Once the bookmark characters are processed this method is called.
+        */
+        virtual void bookmarkEnd();
+
         /**
          * 2.3.4
          *
@@ -238,8 +249,8 @@ namespace wvWare {
         /**
          * This method is called when the image is escher data.
          */
-        virtual void escherData( OLEImageReader& reader, SharedPtr<const Word97::PICF> picf, int type );
-        virtual void escherData( std::vector<U8> data, SharedPtr<const Word97::PICF> picf, int type );
+        virtual void escherData( OLEImageReader& reader, SharedPtr<const Word97::PICF> picf, int type, U32 pib);
+        virtual void escherData( std::vector<U8> data, SharedPtr<const Word97::PICF> picf, int type, U32 pib);
         /**
          * This method is called when you invoke a PictureFunctor and the embedded
          * image is a .wmf file. The data can be accessed using the OLEImageReader.
@@ -385,7 +396,14 @@ namespace wvWare {
         virtual void footnoteFound( FootnoteData::Type type, UString characters,
                                     SharedPtr<const Word97::CHP> chp, const FootnoteFunctor& parseFootnote);
 
-
+        /**
+        * The parser found a bookmark. The passed functor will trigger the parsing of this
+        * bookmark start/end, the default implementation just emits the passed character
+        * with runOfText (that it doesn't get lost if someone doesn't override this method)
+        * and invokes the functor.
+        */
+        virtual void bookmarkFound( UString characters, UString name,
+                                    SharedPtr<const Word97::CHP> chp, const BookmarkFunctor& parseBookmark);
         /**
          * The parser found an annotation. The passed functor will trigger the parsing of this
          * annotation, the default implementation just emits the passed character with
