@@ -26,6 +26,7 @@
 
 #include <KoGenStyle.h>
 #include <styles/KoCharacterStyle.h>
+#include <styles/KoListLevelProperties.h>
 
 #include "MsooXmlReader.h"
 
@@ -51,6 +52,7 @@ protected:
 
     //! Used for creating style in w:pPr (style:style/@style:name attr)
     KoGenStyle m_currentParagraphStyle;
+    bool m_currentParagraphStylePredefined; //!< true if m_currentParagraphStyle shouldn't be created in read_pPr
 
     void setupParagraphStyle();
 
@@ -58,9 +60,15 @@ protected:
     bool m_currentTextStylePredefined; //!< true if m_currentTextStyle shouldn't be created in read_rPr
     KoCharacterStyle* m_currentTextStyleProperties;
 
+    KoGenStyle m_currentListStyle;
+    KoListLevelProperties* m_currentListStyleProperties;
+
     //! Style (from styles.xml) to apply to the current paragraph or similar element, set by read_pStyle()
     QString m_currentStyleName;
 
+    //! Set by rStyle
+    QString m_currentRunStyleName;
+
     bool isDefaultTocStyle(const QString& name) const;
 
     //! Adds reference to a file in the ODF document to manifest.xml
@@ -82,6 +90,8 @@ protected:
 
     bool m_addManifestEntryForPicturesDirExecuted;
 
+    bool m_moveToStylesXml;
+
 //    //! Used for creating style names (style:style/@style:name attr)
 //    //! To achieve this, in XSLT it generate-id(.) for w:p is used.
 //    ////! Starts with 1. Updated in read_p();
