@@ -89,12 +89,24 @@ protected:
         const QByteArray& contentType, MsooXmlReader *reader, KoOdfWriters *writers,
         QString& errorMessage, MsooXmlReaderContext* context = 0);
 
+    //! Like @ref loadAndParseDocument(const QByteArray&, MsooXmlReader*, KoOdfWriters*, QString&, MsooXmlReaderContext*)
+    //! but file name is provided directly instead of content type
+    KoFilter::ConversionStatus loadAndParseDocumentFromFile(
+        const QString& fileName, MsooXmlReader *reader, KoOdfWriters *writers,
+        QString& errorMessage, MsooXmlReaderContext* context);
+
     //! Like @ref loadAndParseDocument(const QByteArray&, MsooXmlReader*, KoOdfWriters*, QString&, MsooXmlReaderContext*)
     //! but return KoFilter::OK if the document for the content type is not found.
     KoFilter::ConversionStatus loadAndParseDocumentIfExists(
         const QByteArray& contentType, MsooXmlReader *reader, KoOdfWriters *writers,
         QString& errorMessage, MsooXmlReaderContext* context = 0);
 
+    //! Like @ref loadAndParseDocumentIfExists(const QByteArray&, MsooXmlReader*, KoOdfWriters*, QString&, MsooXmlReaderContext*)
+    //! but file name is provided directly instead of content type
+    KoFilter::ConversionStatus loadAndParseDocumentFromFileIfExists(
+        const QString& fileName, MsooXmlReader *reader, KoOdfWriters *writers,
+        QString& errorMessage, MsooXmlReaderContext* context = 0);
+
     KoFilter::ConversionStatus parseThemes(QMap<QString, MSOOXML::DrawingMLTheme*>& themes,
         KoOdfWriters *writers, QString& errorMessage);
 
@@ -110,6 +122,10 @@ private:
         const QByteArray& contentType, MsooXmlReader *reader, KoOdfWriters *writers,
         QString& errorMessage, MsooXmlReaderContext* context, bool *pathFound);
 
+    KoFilter::ConversionStatus loadAndParseDocumentFromFileInternal(
+        const QString& fileName, MsooXmlReader *reader, KoOdfWriters *writers,
+        QString& errorMessage, MsooXmlReaderContext* context, bool *pathFound);
+
     KZip* m_zip; //!< Input zip file
 
     KoStore* m_outputStore; //!< output store used for copying files
