@@ -7,6 +7,23 @@
  *
  * Contact: Suresh Chande suresh.chande@nokia.com
  *
+ * Utils::columnName() based on Cell::columnName() from koffice/kspread/Utils.cpp:
+ * Copyright 2006-2007 Stefan Nikolaus <stefan.nikolaus@kdemail.net>
+ * Copyright 2004 Tomas Mecir <mecirt@gmail.com>
+ * Copyright 1999-2002,2004 Laurent Montel <montel@kde.org>
+ * Copyright 2002,2004 Ariya Hidayat <ariya@kde.org>
+ * Copyright 2002-2003 Norbert Andres <nandres@web.de>
+ * Copyright 2003 Stefan Hetzl <shetzl@chello.at>
+ * Copyright 2001-2002 Philipp Mueller <philipp.mueller@gmx.de>
+ * Copyright 2002 Harri Porten <porten@kde.org>
+ * Copyright 2002 John Dailey <dailey@vt.edu>
+ * Copyright 1999-2001 David Faure <faure@kde.org>
+ * Copyright 2000-2001 Werner Trobin <trobin@kde.org>
+ * Copyright 2000 Simon Hausmann <hausmann@kde.org
+ * Copyright 1998-1999 Torben Weis <weis@kde.org>
+ * Copyright 1999 Michael Reiher <michael.reiher@gmx.de>
+ * Copyright 1999 Reginald Stadlbauer <reggie@kde.org>
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
  * version 2.1 as published by the Free Software Foundation.
@@ -932,20 +949,48 @@ void Utils::XmlWriteBuffer::clear()
     m_origWriter = 0;
 }
 
+QString Utils::columnName(uint column)
+{
+    uint digits = 1;
+    uint offset = 0;
+
+    for (uint limit = 26; column >= limit + offset; limit *= 26, digits++)
+        offset += limit;
+
+    QString str;
+    for (uint col = column - offset; digits > 0; --digits, col /= 26)
+        str.prepend(QChar('A' + (col % 26)));
+
+    return str;
+}
+
 // <units> -------------------
 
-QString Utils::EMU_to_ODF(const QString& emuValue)
+QString Utils::EMU_to_ODF(const QString& twipValue)
 {
-    if (emuValue.isEmpty())
+    if (twipValue.isEmpty())
         return QLatin1String("0cm");
     bool ok;
-    const int emu = emuValue.toInt(&ok);
+    const int emu = twipValue.toInt(&ok);
     if (!ok)
         return QString();
     if (emu == 0)
         return QLatin1String("0cm");
     return EMU_TO_CM_STRING(emu);
-//    return QString::number(, 'g', 3) + QLatin1String("cm");
+}
+
+QString Utils::TWIP_to_ODF(const QString& twipValue)
+{
+    if (twipValue.isEmpty())
+        return QLatin1String("0cm");
+    bool ok;
+    const int twip = twipValue.toInt(&ok);
+    if (!ok)
+        return QString();
+    if (twip == 0)
+        return QLatin1String("0cm");
+    QString res;
+    return res.sprintf("%3.3fcm", TWIP_TO_CM(double(twip)));
 }
 
 QString Utils::ST_EighthPointMeasure_to_ODF(const QString& value)
@@ -960,7 +1005,7 @@ QString Utils::ST_EighthPointMeasure_to_ODF(const QString& value)
 }
 
 //! @return true if @a string is non-negative integer number
-static bool isNumber(const QString& string)
+static bool isPositiveIntegerNumber(const QString& string)
 {
     for (const QChar *c = string.constData(); !c->isNull(); c++) {
         if (!c->isNumber())
@@ -970,12 +1015,12 @@ static bool isNumber(const QString& string)
 }
 
 //! Splits number and unit
-static bool splitNumberAndUnit(const QString& _string, int *number, QString* unit)
+static bool splitNumberAndUnit(const QString& _string, qreal *number, QString* unit)
 {
     int unitIndex = 0;
     QString string(_string);
     for (const QChar *c = string.constData(); !c->isNull(); c++, unitIndex++) {
-        if (!c->isNumber())
+        if (!c->isNumber() && *c != '.')
             break;
     }
     *unit = string.mid(unitIndex);
@@ -985,7 +1030,7 @@ static bool splitNumberAndUnit(const QString& _string, int *number, QString* uni
         return false;
     }
     bool ok;
-    *number = string.toUInt(&ok);
+    *number = string.toFloat(&ok);
     if (!ok)
         kWarning() << "Invalid number in" << _string;
     return ok;
@@ -1004,25 +1049,46 @@ static bool isUnitAcceptable(const QString& unit)
            || unit == QString::fromLatin1("pi");
 }
 
-MSOOXML_EXPORT QString Utils::ST_TwipsMeasure_to_ODF(const QString& value)
+static QString ST_TwipsMeasure_to_ODF_with_unit(const QString& value,
+                                                qreal (*convertFromTwips)(qreal), const char* unit)
 {
     if (value.isEmpty())
         return QString();
-    if (isNumber(value)) {
+    if (isPositiveIntegerNumber(value)) {
         // a positive number in twips (twentieths of a point, equivalent to 1/1440th of an inch)
         bool ok;
-        const qreal point = TWIP_TO_POINT( qreal(value.toFloat(&ok)) );
+        const qreal point = convertFromTwips( qreal(value.toFloat(&ok)) );
         if (!ok)
             return QString();
-        return QString::number(point, 'g', 2) + QLatin1String("pt");
+        return QString::number(point, 'g', 2) + QLatin1String(unit);
     }
     return Utils::ST_PositiveUniversalMeasure_to_ODF(value);
 }
 
+qreal twipToPt(qreal v)
+{
+    return TWIP_TO_POINT(v);
+}
+
+MSOOXML_EXPORT QString Utils::ST_TwipsMeasure_to_pt(const QString& value)
+{
+    return ST_TwipsMeasure_to_ODF_with_unit(value, twipToPt, "pt");
+}
+
+qreal twipToCm(qreal v)
+{
+    return TWIP_TO_CM(v);
+}
+
+MSOOXML_EXPORT QString Utils::ST_TwipsMeasure_to_cm(const QString& value)
+{
+    return ST_TwipsMeasure_to_ODF_with_unit(value, twipToCm, "cm");
+}
+
 MSOOXML_EXPORT QString Utils::ST_PositiveUniversalMeasure_to_ODF(const QString& value)
 {
     // a positive decimal number immediately following by a unit identifier.
-    int number;
+    qreal number;
     QString unit;
     if (!splitNumberAndUnit(value, &number, &unit))
         return QString();
@@ -1037,4 +1103,13 @@ MSOOXML_EXPORT QString Utils::ST_PositiveUniversalMeasure_to_ODF(const QString&
     return value; // the original is OK
 }
 
+MSOOXML_EXPORT QString Utils::ST_PositiveUniversalMeasure_to_cm(const QString& value)
+{
+    QString v(ST_PositiveUniversalMeasure_to_ODF(value));
+    if (v.isEmpty())
+        return QString();
+    QString res;
+    return res.sprintf("%3.3fcm", POINT_TO_CM(KoUnit::parseValue(v)));
+}
+
 // </units> -------------------
