@@ -37,7 +37,10 @@
 
 // KDChart
 #include <KDChartDataValueAttributes>
+#include <KDChartPieAttributes>
 #include <KDChartTextAttributes>
+#include <KDChartRelativePosition>
+#include <KDChartPosition>
 #include <KDChartAbstractDiagram>
 #include <KDChartMeasure>
 #include "KDChartModel.h"
@@ -51,11 +54,15 @@
 // KOffice
 #include <KoXmlNS.h>
 #include <KoOdfGraphicStyles.h>
+#include <KoGenStyle.h>
 #include <KoXmlReader.h>
 #include <KoShapeLoadingContext.h>
+#include <KoShapeSavingContext.h>
 #include <KoOdfLoadingContext.h>
 #include <KoOdfWorkaround.h>
-
+#include <KoGenStyle.h>
+#include <KoGenStyles.h>
+#include <KoXmlWriter.h>
 
 using namespace KChart;
 
@@ -89,7 +96,6 @@ public:
     Axis *attachedAxis;
     bool showMeanValue;
     QPen meanValuePen;
-    bool showValues;
     bool showLabels;
     bool showLowerErrorIndicator;
     bool showUpperErrorIndicator;
@@ -105,11 +111,19 @@ public:
     bool brushIsSet;
     QPen pen;
     QBrush brush;
+
+    // Returns an instance of DataValueAttributes with sane default values in
+    // relation to KChart
+    KDChart::DataValueAttributes defaultDataValueAttributes();
+
     KDChart::PieAttributes pieAttributes;
+    KDChart::DataValueAttributes dataValueAttributes;
 
     QMap<int, QPen> pens;
     QMap<int, QBrush> brushes;
     QMap<int, KDChart::PieAttributes> sectionsPieAttributes;
+    QMap<int, KDChart::DataValueAttributes> sectionsDataValueAttributes;
+    QMap<int, bool> sectionsShowLabels;
 
     int num;
 
@@ -142,7 +156,6 @@ DataSet::Private::Private( DataSet *parent )
     kdChartModel = 0;
     kdDataSetNumber = -1;
     showMeanValue = false;
-    showValues = false;
     showLabels = false;
     showLowerErrorIndicator = false;
     showUpperErrorIndicator = false;
@@ -158,12 +171,49 @@ DataSet::Private::Private( DataSet *parent )
     blockSignals = false;
     penIsSet = false;
     brushIsSet = false;
+    dataValueAttributes = defaultDataValueAttributes();
 }
 
 DataSet::Private::~Private()
 {
 }
 
+KDChart::DataValueAttributes DataSet::Private::defaultDataValueAttributes()
+{
+    KDChart::DataValueAttributes attr;
+    KDChart::TextAttributes textAttr = attr.textAttributes();
+    KDChart::Measure fontSize = textAttr.fontSize();
+    fontSize.setValue( 10 );
+    // Don't change font size with chart size
+    fontSize.setCalculationMode( KDChartEnums::MeasureCalculationModeAbsolute );
+    textAttr.setFontSize( fontSize );
+    // Draw text horizontally
+    textAttr.setRotation( 0 );
+    attr.setTextAttributes( textAttr );
+    // Set positive value position
+    KDChart::RelativePosition positivePosition = attr.positivePosition();
+    positivePosition.setAlignment( Qt::AlignCenter | Qt::AlignBottom );
+    positivePosition.setReferencePosition( KDChartEnums::PositionNorth );
+    positivePosition.setHorizontalPadding( 0.0 );
+    positivePosition.setVerticalPadding( -100.0 );
+    attr.setPositivePosition( positivePosition );
+    // Set negative value position
+    KDChart::RelativePosition negativePosition = attr.negativePosition();
+    negativePosition.setAlignment( Qt::AlignCenter | Qt::AlignTop );
+    negativePosition.setReferencePosition( KDChartEnums::PositionSouth );
+    negativePosition.setHorizontalPadding( 0.0 );
+    negativePosition.setVerticalPadding( 100.0 );
+    attr.setNegativePosition( negativePosition );
+    // No decimal digits by default
+    attr.setDecimalDigits( 0 );
+    // Show all values, even if they overlap
+    attr.setShowOverlappingDataLabels( true );
+    // Yes, data point labels can repeatedly have the same text. (e.g. the same value)
+    attr.setShowRepetitiveDataLabels( true );
+
+    return attr;
+}
+
 void DataSet::Private::updateSize()
 {
     int newSize = 0;
@@ -473,40 +523,19 @@ void DataSet::setAttachedAxis( Axis *axis )
     d->attachedAxis = axis;
 }
 
-bool DataSet::showValues() const
-{
-    return d->showValues;
-}
-
-bool DataSet::showLabels() const
+bool DataSet::showLabels( int section /* = -1 */ ) const
 {
+    if ( section >= 0 )
+        return d->sectionsShowLabels[ section ];
     return d->showLabels;
 }
 
-void DataSet::setShowValues( bool showValues )
+void DataSet::setShowLabels( bool showLabels, int section /* = -1 */ )
 {
-    if ( !d->kdDiagram )
-        return;
-
-    d->showValues = showValues;
-    
-    KDChart::DataValueAttributes attributes = d->kdDiagram->dataValueAttributes( d->kdDataSetNumber );
-    attributes.setVisible( showValues );
-
-    // FIXME: This should be a dynamic property that can be changed by
-    //        the user (for 2.1)
-    KDChart::TextAttributes textAttributes = attributes.textAttributes();
-    textAttributes.setFontSize( KDChart::Measure( 6, KDChartEnums::MeasureCalculationModeAbsolute ) );
-    attributes.setTextAttributes( textAttributes );
-    d->kdDiagram->setDataValueAttributes( d->kdDataSetNumber, attributes );
-    
-    if ( d->attachedAxis )
-        d->attachedAxis->update();
-}
-
-void DataSet::setShowLabels( bool showLabels )
-{
-    d->showLabels = showLabels;
+    if ( section >= 0 )
+        d->sectionsShowLabels[ section ] = showLabels;
+    else
+        d->showLabels = showLabels;
 }
 
 QPen DataSet::pen() const
@@ -547,6 +576,13 @@ KDChart::PieAttributes DataSet::pieAttributes( int section ) const
     return pieAttributes();
 }
 
+KDChart::DataValueAttributes DataSet::dataValueAttributes( int section /* = -1 */ ) const
+{
+    if ( d->sectionsDataValueAttributes.contains( section ) )
+        return d->sectionsDataValueAttributes[ section ];
+    return d->dataValueAttributes;
+}
+
 void DataSet::setPen( const QPen &pen )
 {
     d->pen = pen;
@@ -718,7 +754,7 @@ QVariant DataSet::labelData() const
         label += d->data( d->labelDataRegion, i ).toString();
     
     if ( label.isEmpty() )
-        label = i18n( "Series %1", number() );
+        label = i18n( "Series %1", number() + 1 );
 
     return QVariant( label );
 }
@@ -980,49 +1016,83 @@ void DataSet::blockSignals( bool block )
     d->blockSignals = block;
 }
 
+void DataSet::setValueLabelType( ValueLabelType type, int section /* = -1 */ )
+{
+    KDChart::DataValueAttributes &attr = d->dataValueAttributes;
+    if ( section >= 0 && !d->sectionsDataValueAttributes.contains( section ) )
+        d->sectionsDataValueAttributes[ section ] = d->defaultDataValueAttributes();
+    if ( section >= 0 )
+        attr = d->sectionsDataValueAttributes[ section ];
+
+    switch ( type ) {
+        case NoValueLabel:
+            attr.setVisible( false );
+            break;
+        case RealValueLabel:
+            attr.setVisible( true );
+            attr.setUsePercentage( false );
+            break;
+        case PercentageValueLabel:
+            attr.setVisible( true );
+            attr.setUsePercentage( true );
+            attr.setSuffix( "%" );
+            break;
+    }
+}
+
+DataSet::ValueLabelType DataSet::valueLabelType( int section /* = -1 */ ) const
+{
+    KDChart::DataValueAttributes &attr = d->dataValueAttributes;
+    if ( d->sectionsDataValueAttributes.contains( section ) )
+        attr = d->sectionsDataValueAttributes[ section ];
+
+    if ( !attr.isVisible() )
+        return NoValueLabel;
+    if ( !attr.usePercentage() )
+        return RealValueLabel;
+    return PercentageValueLabel;
+}
+
 bool loadBrushAndPen(KoShapeLoadingContext &context, const KoXmlElement &n, QBrush& brush, bool& brushLoaded, QPen& pen, bool& penLoaded)
 {
-    if ( ! n.hasAttributeNS( KoXmlNS::chart, "style-name" ) )
-        return false;
+    if ( n.hasAttributeNS( KoXmlNS::chart, "style-name" ) ) {
+        KoOdfLoadingContext &odfLoadingContext = context.odfLoadingContext();
+        KoStyleStack &styleStack = odfLoadingContext.styleStack();
+        styleStack.save();
+        styleStack.clear();
+        odfLoadingContext.fillStyleStack( n, KoXmlNS::chart, "style-name", "chart" );
 
-    KoOdfLoadingContext &odfLoadingContext = context.odfLoadingContext();
-    KoStyleStack &styleStack = odfLoadingContext.styleStack();
-    styleStack.save();
-    styleStack.clear();
-    odfLoadingContext.fillStyleStack( n, KoXmlNS::chart, "style-name", "chart" );
+        brushLoaded = false;
+        penLoaded = false;
 
-    brushLoaded = false;                                                                                                                                                 
-    penLoaded = false; 
+        styleStack.setTypeProperties( "graphic" );
 
-    styleStack.setTypeProperties( "graphic" );
-
-    if ( styleStack.hasProperty( KoXmlNS::draw, "stroke" ) ) {
-        QString stroke = styleStack.property( KoXmlNS::draw, "stroke" );
-        pen = KoOdfGraphicStyles::loadOdfStrokeStyle( styleStack, stroke, odfLoadingContext.stylesReader() );
-        penLoaded = true;
-    }
-
-    if ( styleStack.hasProperty( KoXmlNS::draw, "fill" ) ) {
-        QString fill = styleStack.property( KoXmlNS::draw, "fill" );
-        if ( fill == "solid" || fill == "hatch" ) {
-            brush = KoOdfGraphicStyles::loadOdfFillStyle( styleStack, fill, odfLoadingContext.stylesReader() );
-            brushLoaded = true;
-        } else if ( fill == "gradient" ) {
-            brush = KoOdfGraphicStyles::loadOdfGradientStyle( styleStack, odfLoadingContext.stylesReader(), QSizeF( 5.0, 60.0 ) );
-            brushLoaded = true;
-        } else if ( fill == "bitmap" ) {
-            brush = Surface::loadOdfPatternStyle( styleStack, odfLoadingContext, QSizeF( 5.0, 60.0 ) );
-            brushLoaded = true;
+        if ( styleStack.hasProperty( KoXmlNS::draw, "stroke" ) ) {
+            QString stroke = styleStack.property( KoXmlNS::draw, "stroke" );
+            pen = KoOdfGraphicStyles::loadOdfStrokeStyle( styleStack, stroke, odfLoadingContext.stylesReader() );
+            penLoaded = true;
         }
+
+        if ( styleStack.hasProperty( KoXmlNS::draw, "fill" ) ) {
+            QString fill = styleStack.property( KoXmlNS::draw, "fill" );
+            if ( fill == "solid" || fill == "hatch" ) {
+                brush = KoOdfGraphicStyles::loadOdfFillStyle( styleStack, fill, odfLoadingContext.stylesReader() );
+                brushLoaded = true;
+            } else if ( fill == "gradient" ) {
+                brush = KoOdfGraphicStyles::loadOdfGradientStyle( styleStack, odfLoadingContext.stylesReader(), QSizeF( 5.0, 60.0 ) );
+                brushLoaded = true;
+            } else if ( fill == "bitmap" ) {
+                brush = Surface::loadOdfPatternStyle( styleStack, odfLoadingContext, QSizeF( 5.0, 60.0 ) );
+                brushLoaded = true;
+            }
+        }
+
+        styleStack.restore();
     }
     
 #ifndef NWORKAROUND_ODF_BUGS
     if( ! penLoaded) {
-        QColor fixedColor = KoOdfWorkaround::fixMissingStrokeColor( n, context );
-        if ( fixedColor.isValid() ) {
-            pen = fixedColor;
-            penLoaded = true;
-        }
+        penLoaded = KoOdfWorkaround::fixMissingStroke( pen, n, context );
     }
     if( ! brushLoaded) {
         QColor fixedColor = KoOdfWorkaround::fixMissingFillColor( n, context );
@@ -1032,8 +1102,6 @@ bool loadBrushAndPen(KoShapeLoadingContext &context, const KoXmlElement &n, QBru
         }
     }
 #endif
-
-    styleStack.restore();
     return true;
 }
 
@@ -1068,6 +1136,19 @@ bool DataSet::loadOdf( const KoXmlElement &n,
         const QString region = n.attributeNS( KoXmlNS::chart, "label-cell-address", QString() );
         setLabelDataRegionString( region );
     }
+    if ( n.hasAttributeNS( KoXmlNS::chart, "data-label-text" ) ) {
+        const QString enable = n.attributeNS( KoXmlNS::chart, "data-label-text", QString() );
+        setShowLabels( enable == "true" );
+    }
+    if ( styleStack.hasProperty(KoXmlNS::chart, "data-label-number" ) ) {
+        const QString format = styleStack.property( KoXmlNS::chart, "data-label-number" );
+        ValueLabelType type = NoValueLabel;
+        if ( format == "value" )
+            type = RealValueLabel;
+        else if ( format == "percentage" )
+            type = PercentageValueLabel;
+        setValueLabelType( type );
+    }
 
     // load data points
     KoXmlElement m;
@@ -1100,3 +1181,31 @@ bool DataSet::loadOdf( const KoXmlElement &n,
 
     return true;
 }
+
+void DataSet::saveOdf( KoShapeSavingContext &context ) const
+{
+    KoXmlWriter &bodyWriter = context.xmlWriter();
+    KoGenStyles &mainStyles = context.mainStyles();
+
+    bodyWriter.startElement( "chart:series" );
+
+    KoGenStyle style( KoGenStyle::ChartAutoStyle, "chart" );
+
+    style.addProperty( "chart:data-label-text", showLabels() ? "true" : "false"  );
+    style.addProperty( "chart:family", ODF_CHARTTYPES[ chartType() ] );
+
+    KoOdfGraphicStyles::saveOdfFillStyle( style, mainStyles, brush() );
+    KoOdfGraphicStyles::saveOdfStrokeStyle( style, mainStyles, pen() );
+
+    const QString styleName = mainStyles.insert( style, "ch" );
+    bodyWriter.addAttribute( "chart:style-name", styleName );
+
+    // TODO: Save external data sources also
+    const QString prefix( "local-table." );
+
+    // Save cell regions
+    bodyWriter.addAttribute( "chart:values-cell-range-address", prefix + yDataRegionString() );
+    bodyWriter.addAttribute( "chart:label-cell-address", prefix + labelDataRegionString() );
+
+    bodyWriter.endElement(); // chart:series
+}
