@@ -34,6 +34,8 @@
 
 // KDChart
 #include <KDChartGlobal>
+#include <KDChartPieAttributes>
+#include <KDChartDataValueAttributes>
 
 using namespace KChart;
 
@@ -41,7 +43,7 @@ class KDChartModel::Private {
 public:
     Private();
     ~Private();
-    
+
     int  dataSetIndex( DataSet *dataSet ) const;
     int  maxDataSetSize() const;
     void updateMaxDataSetSize();
@@ -50,7 +52,7 @@ public:
     int             dataDimensions;
     QList<DataSet*> dataSets;
     int             biggestDataSetIndex;
-    
+
     Qt::Orientation dataDirection;
 };
 
@@ -175,7 +177,9 @@ QVariant KDChartModel::data( const QModelIndex &index,
     case KDChart::DatasetPenRole:
         return dataSet->pen( section );
     case KDChart::PieAttributesRole:
-        return QVariant::fromValue(dataSet->pieAttributes( section ));
+        return QVariant::fromValue( dataSet->pieAttributes( section ) );
+    case KDChart::DataValueLabelAttributesRole:
+        return QVariant::fromValue( dataSet->dataValueAttributes( section ) );
     }
 
     // TODO (Johannes): Support for third data dimension
@@ -203,9 +207,22 @@ void KDChartModel::dataSetChanged( DataSet *dataSet, DataRole role, int first, i
     if ( !d->dataSets.contains( dataSet ) )
         return;
 
+    // be sure the 'first' and 'last' referenced rows are within our boundaries
+    const int rows = rowCount();
+    if ( first >= rows )
+        first = rows - 1;
+    if ( last >= rows )
+        last = rows - 1;
     // 'last' defaults to -1, which means only one column was changed
     if ( last == -1 )
         last = first;
+    // 'first' can be negative either cause rowCount()==0 or cause it still contains the default value of -1. In both cases we abort
+    // and don't progress the update-request future. Same is true for last which should at this point contain a valid row index too.
+    if ( first < 0 || last < 0 )
+        return;
+    // be sure we are not dealing with inverse order
+    if ( last < first )
+        qSwap(first , last);
 
     int dataSetColumn = d->dataSetIndex( dataSet ) * dataDimensions();
 
@@ -214,6 +231,8 @@ void KDChartModel::dataSetChanged( DataSet *dataSet, DataRole role, int first, i
         case YDataRole:
             dataSetColumn += 1;
             break;
+        default:
+            ;
         }
     }
 
@@ -281,7 +300,7 @@ void KDChartModel::dataSetSizeChanged( DataSet *dataSet, int newSize )
     }
 }
 
-void KDChartModel::slotColumnsInserted( const QModelIndex& parent, 
+void KDChartModel::slotColumnsInserted( const QModelIndex& parent,
                                         int start, int end )
 {
     if ( d->dataDirection == Qt::Horizontal ) {
@@ -300,6 +319,7 @@ bool KDChartModel::Private::isKnownDataRole( int role ) const
     case KDChart::DatasetPenRole:
     case KDChart::DatasetBrushRole:
     case KDChart::PieAttributesRole:
+    case KDChart::DataValueLabelAttributesRole:
         return true;
     }
 
@@ -331,12 +351,12 @@ QVariant KDChartModel::headerData( int section,
     if ( !d->isKnownDataRole( role ) ) {
         return QVariant();
     }
-    
+
     if ( d->dataSets.isEmpty() ) {
         qWarning() << "KDChartModel::headerData(): Attempting to request header, but model has no datasets assigned to it.";
         return QVariant();
     }
-    
+
     if ( orientation != d->dataDirection ) {
         int dataSetNumber = section / d->dataDimensions;
         Q_ASSERT( dataSetNumber < d->dataSets.count() );
@@ -351,6 +371,8 @@ QVariant KDChartModel::headerData( int section,
             return dataSet->pen();
         case KDChart::PieAttributesRole:
             return QVariant::fromValue(dataSet->pieAttributes());
+        case KDChart::DataValueLabelAttributesRole:
+            return QVariant::fromValue( dataSet->dataValueAttributes() );
         }
     }
     /* else if ( orientation == d->dataDirection ) { */
@@ -375,7 +397,7 @@ QVariant KDChartModel::headerData( int section,
 }
 
 QModelIndex KDChartModel::index( int row, int column,
-				 const QModelIndex &parent ) const
+                                 const QModelIndex &parent ) const
 {
     Q_UNUSED( parent );
 
@@ -433,14 +455,14 @@ void KDChartModel::addDataSet( DataSet *dataSet, bool silent )
     }
 
     dataSet->setKdChartModel( this );
-    
+
     int dataSetIndex = d->dataSetIndex( dataSet );
-    
+
     if ( silent ) {
-    	d->dataSets.insert( dataSetIndex, dataSet );
-    	const int dataSetSize = dataSet->size();
+        d->dataSets.insert( dataSetIndex, dataSet );
+        const int dataSetSize = dataSet->size();
         if ( d->biggestDataSetIndex < 0 || dataSetSize > d->maxDataSetSize() )
-        	d->biggestDataSetIndex = dataSetIndex;
+                d->biggestDataSetIndex = dataSetIndex;
 
         // Regenerate the numbers for all the datasets.
         for ( int i = dataSetIndex; i < d->dataSets.size(); i++ )
@@ -453,7 +475,7 @@ void KDChartModel::addDataSet( DataSet *dataSet, bool silent )
                                 columnAboutToBeInserted + d->dataDimensions - 1 );
         }
         else
-            beginInsertRows( QModelIndex(), columnAboutToBeInserted, 
+            beginInsertRows( QModelIndex(), columnAboutToBeInserted,
                              columnAboutToBeInserted + d->dataDimensions - 1 );
         d->dataSets.insert( dataSetIndex, dataSet );
 
@@ -492,7 +514,7 @@ void KDChartModel::addDataSet( DataSet *dataSet, bool silent )
         d->dataSets.append( dataSet );
         dataSet->setKdDataSetNumber( 0 );
         d->biggestDataSetIndex = 0;
-		
+
         reset();
     }
 }
@@ -502,12 +524,13 @@ void KDChartModel::removeDataSet( DataSet *dataSet, bool silent )
     const int dataSetIndex = d->dataSets.indexOf( dataSet );
     if ( dataSetIndex < 0 )
         return;
-    
+
     if ( silent ) {
-    	d->dataSets.removeAll( dataSet );
-    	
-    	const int dataSetSize = dataSet->size();
-    	if ( dataSetIndex == d->biggestDataSetIndex ) {
+        d->dataSets.removeAll( dataSet );
+
+        const int dataSetSize = dataSet->size();
+        Q_UNUSED(dataSetSize);
+        if ( dataSetIndex == d->biggestDataSetIndex ) {
             int maxSize = 0;
             for ( int i = 0; i < d->dataSets.size(); i++ ) {
                 const int size = d->dataSets[i]->size();
@@ -522,8 +545,8 @@ void KDChartModel::removeDataSet( DataSet *dataSet, bool silent )
             d->biggestDataSetIndex--;
     }
     else {
-    	const int dataSetSize = dataSet->size();
-    	if ( dataSetIndex == d->biggestDataSetIndex ) {
+        const int dataSetSize = dataSet->size();
+        if ( dataSetIndex == d->biggestDataSetIndex ) {
             int maxSize = 0;
             int biggestDataSetIndex = -1;
             for ( int i = 0; i < d->dataSets.size(); i++ ) {
@@ -536,7 +559,7 @@ void KDChartModel::removeDataSet( DataSet *dataSet, bool silent )
                     biggestDataSetIndex = i;
                 }
             }
-    		
+
             if ( maxSize < dataSetSize ) {
                 if ( d->dataDirection == Qt::Horizontal )
                     beginRemoveColumns( QModelIndex(), maxSize, dataSetSize - 1 );
@@ -552,24 +575,24 @@ void KDChartModel::removeDataSet( DataSet *dataSet, bool silent )
                 d->biggestDataSetIndex = biggestDataSetIndex;
             }
         }
-    	
-    	if ( dataSetIndex < d->biggestDataSetIndex )
-    	    d->biggestDataSetIndex--;
-    	
-    	int columnAboutToBeRemoved = dataSetIndex * d->dataDimensions;
+
+        if ( dataSetIndex < d->biggestDataSetIndex )
+            d->biggestDataSetIndex--;
+
+        int columnAboutToBeRemoved = dataSetIndex * d->dataDimensions;
         if ( d->dataDirection == Qt::Horizontal )
             beginRemoveRows( QModelIndex(), columnAboutToBeRemoved,
                              columnAboutToBeRemoved + d->dataDimensions - 1 );
-    	else
+        else
             beginRemoveColumns( QModelIndex(), columnAboutToBeRemoved,
                                 columnAboutToBeRemoved + d->dataDimensions - 1 );
-    	d->dataSets.removeAt( dataSetIndex );
+        d->dataSets.removeAt( dataSetIndex );
         if ( d->dataDirection == Qt::Horizontal )
             endRemoveRows();
         else
             endRemoveColumns();
     }
-    
+
     for ( int i = dataSetIndex; i < d->dataSets.size(); i++ ) {
         d->dataSets[ i ]->setKdDataSetNumber( i );
     }
