@@ -74,7 +74,8 @@
 #include "DataSet.h"
 #include "ThreeDScene.h"
 #include "ChartProxyModel.h"
-
+#include "ScreenConversions.h"
+#include "Layout.h"
 
 using namespace KChart;
 
@@ -208,8 +209,8 @@ void PlotArea::Private::initAxes()
     xAxis->setPosition( BottomAxisPosition );
     yAxis->setPosition( LeftAxisPosition );
     yAxis->setShowMajorGrid( true );
-    axes.append( xAxis );
-    axes.append( yAxis );
+    q->addAxis( xAxis );
+    q->addAxis( yAxis );
 }
 
 PlotArea::PlotArea( ChartShape *parent )
@@ -481,16 +482,29 @@ void PlotArea::setChartType( ChartType type )
         d->automaticallyHiddenAxisTitles.clear();
     }
 
-    // FIXME BUG: This seems buggy.
-    // FIXME NOW: Make this a switch (see comment below)
     // Set the dimensionality of the data points.
-    if ( d->chartType != ScatterChartType && type == ScatterChartType ) {
-        d->shape->proxyModel()->setDataDimensions( 2 );
-    }
-    // FIXME: This is not really true.  Could also be 3 (bubble charts)
-    else if ( d->chartType == ScatterChartType && type != ScatterChartType ) {
-        d->shape->proxyModel()->setDataDimensions( 1 );
+    int dimensions = 1;
+    switch ( type ) {
+        case BarChartType:
+        case LineChartType:
+        case AreaChartType:
+        case CircleChartType:
+        case RingChartType:
+        case RadarChartType:
+        case StockChartType:
+        case GanttChartType:
+            dimensions = 1;
+            break;
+        case ScatterChartType:
+        case SurfaceChartType:
+            dimensions = 2;
+            break;
+        case BubbleChartType:
+            dimensions = 3;
+            break;
     }
+
+    d->shape->proxyModel()->setDataDimensions( dimensions );
     
     d->chartType = type;
     
@@ -600,6 +614,13 @@ bool PlotArea::loadOdf( const KoXmlElement &plotAreaElement,
         styleStack.setTypeProperties( "graphic" );
         styleStack.setTypeProperties( "chart" );
 
+        if ( styleStack.hasProperty( KoXmlNS::chart, "angle-offset" ) ) {
+            bool ok;
+            const int angleOffset = styleStack.property( KoXmlNS::chart, "angle-offset" ).toInt(&ok);
+            if (ok)
+                setPieAngleOffset( angleOffset );
+        }
+
         if ( styleStack.hasProperty( KoXmlNS::chart, "three-dimensional" ) )
             setThreeD( styleStack.property( KoXmlNS::chart, "three-dimensional" ) == "true" );
 
@@ -646,13 +667,23 @@ bool PlotArea::loadOdf( const KoXmlElement &plotAreaElement,
         // documents created by OOo.
         proxyModel()->setDataDirection( Qt::Vertical );
 
+    // The exact position defined in ODF overwrites the default layout position
+    if ( plotAreaElement.hasAttributeNS( KoXmlNS::svg, "x" ) ||
+         plotAreaElement.hasAttributeNS( KoXmlNS::svg, "y" ) ||
+         plotAreaElement.hasAttributeNS( KoXmlNS::svg, "width" ) ||
+         plotAreaElement.hasAttributeNS( KoXmlNS::svg, "height" ) )
+        parent()->layout()->setPosition( this, FloatingPosition );
+
     loadOdfAttributes( plotAreaElement, context, OdfAllAttributes );
     
     // Find out if the data table contains labels as first row and/or column.
     // This is in the plot-area element itself.
-    if( proxyModel()->categoryDataRegion().isEmpty() && // if chart:categories with a table:cell-range-address is defined within an axis then we need to ignore the data-source-has-labels.
-        plotAreaElement.hasAttributeNS( KoXmlNS::chart, "data-source-has-labels" ) ) {
-        // Yes, it does.  Now find out how.
+
+    // Do not ignore the data-source-has-labels in any case, even if a category data region is specified for an axis, as the first column still
+    // has to be exluded from the actual data region if e.g. data-source-has-labels is set to "column"
+    // If an axis contains the chart:categories element, the category data region will automatically be set on every data set attached to that
+    // axis. See Axis::attachDataSet().
+    if ( plotAreaElement.hasAttributeNS( KoXmlNS::chart, "data-source-has-labels" ) ) {
         const QString  dataSourceHasLabels
             = plotAreaElement.attributeNS( KoXmlNS::chart,
                                         "data-source-has-labels" );
@@ -801,9 +832,6 @@ void PlotArea::saveOdf( KoShapeSavingContext &context ) const
     //if ( d->floor )
     //    d->floor->saveOdf( context, "chart:floor" );
 
-    // TODO chart:series
-    // TODO chart:grid
-    
     bodyWriter.endElement(); // chart:plot-area
 }
 
@@ -1072,7 +1100,11 @@ void PlotArea::paint( QPainter& painter, const KoViewConverter& converter )
         d->lastSize      = size();
     }*/
     painter.setRenderHint( QPainter::Antialiasing, false );
-    d->kdChart->paint( &painter, paintRect.toRect() );
+
+    // KDChart thinks in pixels, KOffice in pt
+    ScreenConversions::scaleFromPtToPx( painter );
+
+    d->kdChart->paint( &painter, ScreenConversions::scaleFromPtToPx( paintRect ) );
     //painter.restore();
 
     // Paint the cached pixmap if we got a GO from paintPixmap()
