@@ -62,7 +62,7 @@
 // KOffice
 #include <KoCanvasBase.h>
 #include <KoCanvasController.h>
-#include <KoCanvasResourceProvider.h>
+#include <KoResourceManager.h>
 #include <KoViewConverter.h>
 
 // KDE
@@ -94,20 +94,33 @@ void CellToolBase::Private::updateEditor(const Cell& cell)
 void CellToolBase::Private::updateLocationComboBox()
 {
     QString address;
-    if (q->selection()->activeSheet()->getLcMode()) {
-        if (q->selection()->isSingular()) {
-            address = 'L' + QString::number(q->selection()->marker().y()) +
-                      'C' + QString::number(q->selection()->marker().x());
+    const QList< QPair<QRectF, QString> > names = q->selection()->activeSheet()->cellStorage()->namedAreas(*q->selection());
+    {
+        QRect range;
+        if (q->selection()->isSingular()) range = QRect(q->selection()->marker(), QSize(1, 1));
+        else range = q->selection()->lastRange();
+        for (int i = 0; i < names.size(); i++) {
+            if (names[i].first.toRect() == range) {
+                address = names[i].second;
+            }
+        }
+    }
+    if (address.isEmpty()) {
+        if (q->selection()->activeSheet()->getLcMode()) {
+            if (q->selection()->isSingular()) {
+                address = 'L' + QString::number(q->selection()->marker().y()) +
+                        'C' + QString::number(q->selection()->marker().x());
+            } else {
+                const QRect lastRange = q->selection()->lastRange();
+                address = QString::number(lastRange.bottom() - lastRange.top() + 1) + "Lx";
+                if (Region::Range(lastRange).isRow())
+                    address += QString::number(q->maxCol() - lastRange.left() + 1) + 'C';
+                else
+                    address += QString::number(lastRange.right() - lastRange.left() + 1) + 'C';
+            }
         } else {
-            const QRect lastRange = q->selection()->lastRange();
-            address = QString::number(lastRange.bottom() - lastRange.top() + 1) + "Lx";
-            if (Region::Range(lastRange).isRow())
-                address += QString::number(q->maxCol() - lastRange.left() + 1) + 'C';
-            else
-                address += QString::number(lastRange.right() - lastRange.left() + 1) + 'C';
+            address = q->selection()->name();
         }
-    } else {
-        address = q->selection()->name();
     }
     if (address != locationComboBox->lineEdit()->text())
         locationComboBox->lineEdit()->setText(address);
@@ -130,18 +143,7 @@ void CellToolBase::Private::updateActions(const Cell& cell)
     ACTION_EXEC("underline", setChecked(style.underline()));
     ACTION_EXEC("strikeOut", setChecked(style.strikeOut()));
 
-    // workaround for bug #59291 (crash upon starting from template)
-    // certain Qt and Fontconfig combination fail miserably if can not
-    // find the font name (e.g. not installed in the system)
-    QStringList fontList;
-    KFontChooser::getFontList(fontList, 0);
-    QString fontFamily = style.fontFamily();
-    for (QStringList::Iterator it = fontList.begin(); it != fontList.end(); ++it) {
-        if ((*it).toLower() == fontFamily.toLower()) {
-            static_cast<KFontAction*>(q->action("font"))->setFont(fontFamily);
-            break;
-        }
-    }
+    static_cast<KFontAction*>(q->action("font"))->setFont(style.fontFamily());
     static_cast<KFontSizeAction*>(q->action("fontSize"))->setFontSize(style.fontSize());
     // -- horizontal alignment actions --
     ACTION_EXEC("alignLeft", setChecked(style.halign() == Style::Left));
@@ -847,10 +849,10 @@ bool CellToolBase::Private::formatKeyPress(QKeyEvent * _ev)
 
     case Qt::Key_Ampersand:
         command->setText(i18n("Change Border"));
-        command->setTopBorderPen(QPen(q->canvas()->resourceProvider()->foregroundColor().toQColor(), 1, Qt::SolidLine));
-        command->setBottomBorderPen(QPen(q->canvas()->resourceProvider()->foregroundColor().toQColor(), 1, Qt::SolidLine));
-        command->setLeftBorderPen(QPen(q->canvas()->resourceProvider()->foregroundColor().toQColor(), 1, Qt::SolidLine));
-        command->setRightBorderPen(QPen(q->canvas()->resourceProvider()->foregroundColor().toQColor(), 1, Qt::SolidLine));
+        command->setTopBorderPen(QPen(q->canvas()->resourceManager()->foregroundColor().toQColor(), 1, Qt::SolidLine));
+        command->setBottomBorderPen(QPen(q->canvas()->resourceManager()->foregroundColor().toQColor(), 1, Qt::SolidLine));
+        command->setLeftBorderPen(QPen(q->canvas()->resourceManager()->foregroundColor().toQColor(), 1, Qt::SolidLine));
+        command->setRightBorderPen(QPen(q->canvas()->resourceManager()->foregroundColor().toQColor(), 1, Qt::SolidLine));
         break;
 
     default:
@@ -1190,7 +1192,7 @@ QList<QAction*> CellToolBase::Private::popupActionList() const
             actions.append(popupMenuActions["separator3"]);
             actions.append(popupMenuActions["insertCell"]);
             actions.append(popupMenuActions["deleteCell"]);
-        } else if (q->selection()->isColumnSelected() && q->selection()->isSingular()) {
+        } else if (q->selection()->isColumnSelected()) {
             actions.append(q->action("resizeCol"));
             actions.append(popupMenuActions["adjustColumn"]);
             actions.append(popupMenuActions["separator4"]);
@@ -1227,7 +1229,7 @@ QList<QAction*> CellToolBase::Private::popupActionList() const
                     break;
                 }
             }
-        } else if (q->selection()->isRowSelected() && q->selection()->isSingular()) {
+        } else if (q->selection()->isRowSelected()) {
             actions.append(q->action("resizeRow"));
             actions.append(popupMenuActions["adjustRow"]);
             actions.append(popupMenuActions["separator5"]);
