@@ -23,11 +23,8 @@
 
 #include "KWOdfWriter.h"
 #include "KWDocument.h"
-#include <rdf/KoDocumentRdfBase.h>
 #include "KWPage.h"
 
-#include <ktemporaryfile.h>
-
 #include "frames/KWTextFrameSet.h"
 #include "frames/KWTextFrame.h"
 #include <KoXmlWriter.h>
@@ -44,10 +41,12 @@
 #include <KoTextSharedSavingData.h>
 
 #include <KoStoreDevice.h>
+#include <rdf/KoDocumentRdfBase.h>
 
 #include <QBuffer>
 #include <QTextCursor>
 #include <KDebug>
+#include <ktemporaryfile.h>
 
 QByteArray KWOdfWriter::serializeHeaderFooter(KoEmbeddedDocumentSaver &embeddedSaver, KoGenStyles &mainStyles, KoGenChanges  &changes, KWTextFrameSet *fs)
 {
@@ -76,7 +75,7 @@ QByteArray KWOdfWriter::serializeHeaderFooter(KoEmbeddedDocumentSaver &embeddedS
     Q_ASSERT(shapedata);
 
     writer.startElement(tag);
-    shapedata->saveOdf(context, 0, -1);
+    shapedata->saveOdf(context, 0, -1, m_document->documentRdfBase());
     writer.endElement();
 
     return content;
@@ -241,8 +240,13 @@ bool KWOdfWriter::save(KoOdfWriteStore &odfStore, KoEmbeddedDocumentSaver &embed
         //  3) frames that are not anchored but freely positioned somewhere on the page.
         //     in ODF terms those frames are page-anchored.
 
-        if (fs->frameCount() == 1 && fs->frames().first()->shape()->parent())
-            continue; // is a frame that is anchored to text, don't save those here.
+        if (fs->frameCount() == 1) {
+            KoShape *shape = fs->frames().first()->shape();
+            // may be a frame that is anchored to text, don't save those here.
+            // but first check since clipped shapes look similar, but are not anchored to text
+            if (shape->parent() && !shape->parent()->childClipped(shape))
+                continue;
+        }
 
         KWTextFrameSet *tfs = dynamic_cast<KWTextFrameSet*>(fs);
         if (tfs) {
@@ -367,7 +371,7 @@ bool KWOdfWriter::save(KoOdfWriteStore &odfStore, KoEmbeddedDocumentSaver &embed
                     tbf.setProperty(KoParagraphStyle::MasterPageName, m_masterPages.value(style));
                     cursor.mergeBlockFormat(tbf);
                 }
-                shapeData->saveOdf(context);
+                shapeData->saveOdf(context, 0, -1, m_document->documentRdfBase());
             }
         }
     }
