@@ -1,6 +1,6 @@
 /* This file is part of the KDE project
    Copyright (C) 2006 Casper Boemann Rasmussen <cbr@boemann.dk>
-   Copyright (C) 2006-2009 Thomas Zander <zander@kde.org>
+   Copyright (C) 2006-2010 Thomas Zander <zander@kde.org>
    Copyright (C) 2006-2010 Thorsten Zachmann <zachmann@kde.org>
    Copyright (C) 2007-2009 Jan Hambrecht <jaham@gmx.net>
 
@@ -159,9 +159,8 @@ void KoShapePrivate::removeShapeManager(KoShapeManager *manager)
     shapeManagers.remove(manager);
 }
 // static
-QString KoShapePrivate::getStyleProperty(const char *property, const KoXmlElement &element, KoShapeLoadingContext &context)
+QString KoShapePrivate::getStyleProperty(const char *property, KoShapeLoadingContext &context)
 {
-    Q_UNUSED(element);
     KoStyleStack &styleStack = context.odfLoadingContext().styleStack();
     QString value;
 
@@ -933,21 +932,40 @@ QString KoShape::saveStyle(KoGenStyle &style, KoShapeSavingContext &context) con
     }
 
     QString value;
-    if (isGeometryProtected())
+    if (isGeometryProtected()) {
         value = "position size";
+    }
     if (isContentProtected()) {
         if (! value.isEmpty())
             value += ' ';
         value += "content";
     }
-    if (!value.isEmpty())
-        (const_cast<KoShapePrivate*>(d))->additionalStyleAttributes.insert("style:protect", value);
+    if (!value.isEmpty()) {
+        style.addProperty("style:protect", value);
+    }
 
     QMap<QByteArray, QString>::const_iterator it(d->additionalStyleAttributes.constBegin());
     for (; it != d->additionalStyleAttributes.constEnd(); ++it) {
         style.addProperty(it.key(), it.value());
     }
 
+    if (parent() && parent()->childClipped(this)) {
+        /*
+         * In KOffice clipping is done using a parent shape which can be rotated, sheared etc
+         * and even non-square.  So the ODF interoperability version we write here is really
+         * just a very simple version of that...
+         */
+        qreal top = -position().y();
+        qreal left = -position().x();
+        qreal right = parent()->size().width() - size().width() - left;
+        qreal bottom = parent()->size().height() - size().height() - top;
+
+        style.addProperty("fo:clip", QString("rect(%1pt, %2pt, %3pt, %4pt)")
+                .arg(top, 10, 'f').arg(right, 10, 'f')
+                .arg(bottom, 10, 'f').arg(left, 10, 'f'));
+
+    }
+
     return context.mainStyles().insert(style, context.isSet(KoShapeSavingContext::PresentationShape) ? "pr" : "gr");
 }
 
@@ -979,9 +997,13 @@ void KoShape::loadStyle(const KoXmlElement &element, KoShapeLoadingContext &cont
         delete d->shadow;
         d->shadow = 0;
     }
-    setBackground(loadOdfFill(element, context));
+    setBackground(loadOdfFill(context));
     setBorder(loadOdfStroke(element, context));
-    setShadow(loadOdfShadow(element, context));
+    setShadow(d->loadOdfShadow(context));
+
+    QString protect(styleStack.property(KoXmlNS::style, "protect"));
+    setGeometryProtected(protect.contains("position") || protect.contains("size"));
+    setContentProtected(protect.contains("content"));
 
     styleStack.restore();
 }
@@ -1071,9 +1093,9 @@ bool KoShape::loadOdfAttributes(const KoXmlElement &element, KoShapeLoadingConte
     return true;
 }
 
-KoShapeBackground *KoShape::loadOdfFill(const KoXmlElement &element, KoShapeLoadingContext &context) const
+KoShapeBackground *KoShape::loadOdfFill(KoShapeLoadingContext &context) const
 {
-    QString fill = KoShapePrivate::getStyleProperty("fill", element, context);
+    QString fill = KoShapePrivate::getStyleProperty("fill", context);
     KoShapeBackground *bg = 0;
     if (fill == "solid" || fill == "hatch") {
         bg = new KoColorBackground();
@@ -1100,7 +1122,7 @@ KoShapeBorderModel *KoShape::loadOdfStroke(const KoXmlElement &element, KoShapeL
     KoStyleStack &styleStack = context.odfLoadingContext().styleStack();
     KoOdfStylesReader &stylesReader = context.odfLoadingContext().stylesReader();
 
-    QString stroke = KoShapePrivate::getStyleProperty("stroke", element, context);
+    QString stroke = KoShapePrivate::getStyleProperty("stroke", context);
     if (stroke == "solid" || stroke == "dash") {
         QPen pen = KoOdfGraphicStyles::loadOdfStrokeStyle(styleStack, stroke, stylesReader);
 
@@ -1122,15 +1144,33 @@ KoShapeBorderModel *KoShape::loadOdfStroke(const KoXmlElement &element, KoShapeL
         border->setLineStyle(pen.style(), pen.dashPattern());
 
         return border;
-    } else {
-        return 0;
+#ifndef NWORKAROUND_ODF_BUGS
+    } else if (stroke.isEmpty()) {
+        QPen pen;
+        if (KoOdfWorkaround::fixMissingStroke(pen, element, context)) {
+            KoLineBorder *border = new KoLineBorder();
+
+            // FIXME: (make it possible to) use a cosmetic pen
+            if (pen.widthF() == 0.0)
+                border->setLineWidth(0.5);
+            else
+                border->setLineWidth(pen.widthF());
+            border->setJoinStyle(pen.joinStyle());
+            border->setLineStyle(pen.style(), pen.dashPattern());
+            border->setLineBrush(pen.brush());
+
+            return border;
+        }
+#endif
     }
+
+    return 0;
 }
 
-KoShapeShadow *KoShape::loadOdfShadow(const KoXmlElement &element, KoShapeLoadingContext &context) const
+KoShapeShadow *KoShapePrivate::loadOdfShadow(KoShapeLoadingContext &context) const
 {
     KoStyleStack &styleStack = context.odfLoadingContext().styleStack();
-    QString shadowStyle = KoShapePrivate::getStyleProperty("shadow", element, context);
+    QString shadowStyle = KoShapePrivate::getStyleProperty("shadow", context);
     if (shadowStyle == "visible" || shadowStyle == "hidden") {
         KoShapeShadow *shadow = new KoShapeShadow();
         QColor shadowColor(styleStack.property(KoXmlNS::draw, "shadow-color"));
@@ -1265,7 +1305,17 @@ void KoShape::saveOdfAttributes(KoShapeSavingContext &context, int attributes) c
     }
 
     if (attributes & OdfSize) {
-        const QSizeF s(size());
+        QSizeF s(size());
+        if (parent() && parent()->childClipped(this)) { // being clipped shrinks our visible size
+            // clipping in ODF is done using a combination of visual size and content cliprect.
+            // A picture of 10cm x 10cm displayed in a box of 2cm x 4cm will be scaled (out
+            // of proportion in this case).  If we then add a fo:clip like;
+            // fo:clip="rect(2cm, 3cm, 4cm, 5cm)" (top, right, bottom, left)
+            // our original 10x10 is clipped to 2cm x 4cm  and *then* fitted in that box.
+
+            // TODO do this properly by subtracting rects
+            s = parent()->size();
+        }
         context.xmlWriter().addAttributePt("svg:width", s.width());
         context.xmlWriter().addAttributePt("svg:height", s.height());
     }
@@ -1323,7 +1373,7 @@ void KoShape::saveOdfCommonChildElements(KoShapeSavingContext &context) const
         context.xmlWriter().endElement();
     }
 
-    // save glue points see ODF 9.2.19 Glue Points
+    // TODO: save glue points see ODF 9.2.19 Glue Points
 }
 
 // end loading & saving methods
