@@ -32,6 +32,7 @@
 class KoShapeLoadingContext;
 class QTextCursor;
 class KoBookmarkManager;
+class KoDocumentRdfBase;
 
 /**
  * The KoTextLoader loads is use to load text for one and only one textdocument or shape
@@ -59,10 +60,12 @@ public:
 
     /**
     * Constructor.
+    * Notice that despite this being a QObject there is no 'parent' available for
+    * memory management here.
     *
     * @param context The context the KoTextLoader is called in
     */
-    explicit KoTextLoader(KoShapeLoadingContext &context);
+    explicit KoTextLoader(KoShapeLoadingContext &context, KoDocumentRdfBase *rdfData = 0);
 
     /**
     * Destructor.
@@ -75,7 +78,7 @@ public:
     * This method got called e.g. at the \a KoTextShapeData::loadOdf() method if a TextShape
     * instance likes to load an ODF element.
     */
-    void loadBody(const KoXmlElement &element, QTextCursor &cursor);
+    void loadBody(const KoXmlElement &element, QTextCursor &cursor, bool isDeleteChange = false);
 
 signals:
 
@@ -100,7 +103,7 @@ private:
     /**
     * Load the list from the \p element into the \p cursor .
     */
-    void loadList(const KoXmlElement &element, QTextCursor &cursor);
+    void loadList(const KoXmlElement &element, QTextCursor &cursor, bool isDeleteChange = false);
 
     /**
     * Load the section from the \p element into the \p cursor .
@@ -164,6 +167,26 @@ private:
     */
     void storeDeleteChanges(KoXmlElement &tag);
 
+    /**
+    * Checks whether a list should be treated as a valid list. A Delete change might have a list but should not be treated as such.
+    * This Information is obtained from the ODF Metadata
+    * **This is a work-around for a bug in the spec**
+    */
+    bool isValidList(const QString& xmlId) const;
+
+    /**
+    * Checks whether a list-item should be treated as a valid list-item. A Delete change might have a list-item but should not be treated as such.
+    * This Information is obtained from the ODF Metadata
+    * **This is a work-around for a bug in the spec**
+    */
+    bool isValidListItem(const QString& xmlId) const;
+
+    /**
+    * Find the level of a list from the Metadata.
+    * **This is a work-around for a bug in the spec**
+    */
+    int listLevel(const QString& xmlId) const;
+
     /**
      * This is called in loadSpan to allow Cut and Paste of bookmarks. This
      * method gives a correct, unique, name, respecting the fact that an
