@@ -26,6 +26,7 @@
 #include "TableLayout.h"
 #include "ListItemsHelper.h"
 #include "TextShape.h"
+#include "ToCGenerator.h"
 
 #include <KoTextDocumentLayout.h>
 #include <KoTextShapeData.h>
@@ -72,13 +73,15 @@ Layout::Layout(KoTextDocumentLayout *parent)
         m_endOfDemoText(false),
         m_defaultTabSizing(MM_TO_POINT(15)),
         m_currentTabStop(0),
-        m_dropCapsNChars(0), m_dropCapsAffectsNMoreLines(0),
+        m_dropCapsNChars(0),
+        m_dropCapsAffectsNMoreLines(0),
         m_dropCapsAffectedLineWidthAdjust(0),
         m_y_justBelowDropCaps(0),
+        m_dropCapsPositionAdjust(0),
         m_restartingAfterTableBreak(false),
         m_restartingFirstCellAfterTableBreak(false)
-        ,m_imageCollection(0)
 {
+    m_frameStack.reserve(5); // avoid reallocs
 }
 
 bool Layout::start()
@@ -105,7 +108,7 @@ void Layout::reset()
     m_reset = true;
 }
 
-bool Layout::interrupted()
+bool Layout::isInterrupted() const
 {
     return m_reset;
 }
@@ -122,7 +125,8 @@ qreal Layout::width()
         ptWidth -= m_blockData->counterWidth() + m_blockData->counterSpacing();
     ptWidth -= m_format.leftMargin() + m_format.rightMargin();
     ptWidth -= m_borderInsets.left + m_borderInsets.right + m_shapeBorder.right;
-    ptWidth -= m_dropCapsAffectedLineWidthAdjust;
+    if (m_block.layout()->lineCount() > 1)
+        ptWidth -= m_dropCapsAffectedLineWidthAdjust;
     return ptWidth;
 }
 
@@ -134,7 +138,8 @@ qreal Layout::x()
     }
     result += m_isRtl ? m_format.rightMargin() : (m_format.leftMargin() + listIndent());
     result += m_borderInsets.left + m_shapeBorder.left;
-    result += m_dropCapsAffectedLineWidthAdjust;
+    if (m_block.layout()->lineCount() > 1)
+        result += m_dropCapsAffectedLineWidthAdjust;
     return result;
 }
 
@@ -184,14 +189,12 @@ bool Layout::addLine(QTextLine &line)
     }
 
     qreal height = m_format.doubleProperty(KoParagraphStyle::FixedLineHeight);
-    if (line.textLength() == 1 && m_block.text().at(line.textStart()) == QChar::ObjectReplacementCharacter) {
-        // thats an anchor then, anchors takes no height.
+    if (line.textLength() == 1 && m_block.text().at(line.textStart()) == QChar::ObjectReplacementCharacter && line.descent() == 0.0) {
+        // thats an (non inline) anchor then, anchors takes no height.
         height = 0.1;
     }
     qreal objectHeight = 0.0;
     bool useFixedLineHeight = height != 0.0;
-    if (m_dropCapsNChars > 0)
-        height = line.ascent();
     if (useFixedLineHeight) {
         // QTextLine has its position at the top of the line. So if the ascent changes between lines in a parag
         // because of different font sizes, for example, we have to adjust the position to make the fixed
@@ -238,6 +241,9 @@ bool Layout::addLine(QTextLine &line)
     const qreal footnoteHeight = findFootnote(line, &oldFootnoteDocLength);
 
     if (!m_newShape
+            // In case the text shape automatically resizes itself to fit all contents, every line will fit,
+            // so just lay them out (i.e. do not return true)
+            && m_parent->resizeMethod() == KoTextDocument::NoResize
             // line does not fit.
             && m_data->documentOffset() + shape->size().height() - footnoteHeight
               < m_y + line.height() + m_shapeBorder.bottom
@@ -304,10 +310,8 @@ bool Layout::addLine(QTextLine &line)
     if (m_dropCapsNChars > 0) { // we just laid out the dropped chars
         m_y_justBelowDropCaps = m_y; // save the y position just below the dropped characters
         m_y = line.y();              // keep the same y for the next line
+        line.setPosition(line.position() - QPointF(0, m_dropCapsPositionAdjust));
         m_dropCapsNChars = 0;
-        qreal dropCapDistance = m_format.doubleProperty(KoParagraphStyle::DropCapsDistance);
-        // save the x-offset that the dropped characters have caused on drop-cap-affected lines
-        m_dropCapsAffectedLineWidthAdjust = line.naturalTextWidth() + dropCapDistance + 1;
     } else if (m_dropCapsAffectsNMoreLines > 0) { // we just laid out a drop-cap-affected line
         m_dropCapsAffectsNMoreLines--;
         if (m_dropCapsAffectsNMoreLines == 0) {   // no more drop-cap-affected lines
@@ -330,6 +334,7 @@ bool Layout::nextParag()
     if (layout && !m_restartingFirstCellAfterTableBreak) { // guard against first time or first time after table relayout
         layout->endLayout();
         m_block = m_block.next();
+        updateFrameStack();
         if (m_endOfDemoText) {
             layout = 0;
             m_blockData = 0;
@@ -443,7 +448,7 @@ bool Layout::nextParag()
     m_y += topMargin();
     layout = m_block.layout();
     QTextOption option = layout->textOption();
-    option.setWrapMode(QTextOption::WrapAtWordBoundaryOrAnywhere);
+    option.setWrapMode(m_parent->resizeMethod() == KoTextDocument::NoResize ? QTextOption::WrapAtWordBoundaryOrAnywhere : QTextOption::NoWrap);
     qreal tabStopDistance =  m_format.property(KoParagraphStyle::TabStopDistance).toDouble();
     if (tabStopDistance > 0)
         option.setTabStop(tabStopDistance * qt_defaultDpiY() / 72.);
@@ -454,10 +459,13 @@ bool Layout::nextParag()
     QList<QTextOption::Tab> tabs;
     QVariant variant = m_format.property(KoParagraphStyle::TabPositions);
     if (!variant.isNull()) {
+        const qreal tabOffset = x();
         foreach(const QVariant &tv, qvariant_cast<QList<QVariant> >(variant)) {
             KoText::Tab koTab = tv.value<KoText::Tab>();
             QTextOption::Tab tab;
-            tab.position = koTab.position * qt_defaultDpiY() / 72.; // convertion here is required because Qt thinks in device units and we don't
+
+            // convertion here is required because Qt thinks in device units and we don't
+            tab.position = (koTab.position - tabOffset) * qt_defaultDpiY() / 72.;
             tab.type = koTab.type;
             tab.delimiter = koTab.delimiter;
             tabs.append(tab);
@@ -503,20 +511,55 @@ bool Layout::nextParag()
         QTextCursor blockStart(m_block);
         QTextLayout::FormatRange dropCapsFormatRange;
         dropCapsFormatRange.format = blockStart.charFormat();
-        qreal size;
-        if (blockStart.charFormat().hasProperty(QTextFormat::FontPointSize))
-            size = blockStart.charFormat().fontPointSize();
-        else
-            size = 12.0; // default when unset
-        dropCapsFormatRange.format.setFontPointSize(size * dropCapsLines);
-        dropCapsFormatRange.format.setProperty(KoCharacterStyle::DropCapsAdditionalFormatting, (QVariant) true);
+
+        // find out lineHeight for this block.
+        qreal lineHeight = m_format.doubleProperty(KoParagraphStyle::FixedLineHeight);
+        qreal dropCapsHeight = 0;
+        if (lineHeight == 0) {
+            lineHeight = blockStart.charFormat().fontPointSize();
+            qreal linespacing = m_format.doubleProperty(KoParagraphStyle::LineSpacing);
+            if (linespacing == 0) { // unset
+                int percent = m_format.intProperty(KoParagraphStyle::PercentLineHeight);
+                if (percent != 0)
+                    linespacing = lineHeight * ((percent - 100) / 100.0);
+                else if (linespacing == 0)
+                    linespacing = lineHeight * 0.2; // default
+            }
+            dropCapsHeight = linespacing * (dropCapsLines-1);
+        }
+        const qreal minimum = m_format.doubleProperty(KoParagraphStyle::MinimumLineHeight);
+        if (minimum > 0.0)
+            lineHeight = qMax(lineHeight, minimum);
+
+        dropCapsHeight += lineHeight * dropCapsLines;
+
+        QFont f(blockStart.charFormat().font(), m_parent->paintDevice());
+        QString dropCapsText(m_block.text().left(dropCapsLength));
+        f.setPointSizeF(dropCapsHeight);
+        QChar lastChar = dropCapsText.at(dropCapsText.length()-1);
+        for (int i=0; i < 5; ++i) {
+            QFontMetricsF fm(f);
+            QRectF rect = fm.tightBoundingRect(dropCapsText);
+            m_dropCapsAffectedLineWidthAdjust = rect.width();
+            m_dropCapsAffectedLineWidthAdjust += fm.rightBearing(lastChar);
+            const qreal diff = dropCapsHeight - rect.height();
+            m_dropCapsPositionAdjust = rect.top() + fm.ascent();
+            if (qAbs(diff < 0.5)) // good enough
+                break;
+
+            const qreal adjustment = diff * ((f.pointSizeF() / diff) / (rect.height() / diff));
+            // kDebug() << "adjusting with" << adjustment;
+            f.setPointSizeF(f.pointSizeF() + adjustment);
+        }
+        dropCapsFormatRange.format.setFontPointSize(f.pointSizeF());
+        dropCapsFormatRange.format.setProperty(KoCharacterStyle::DropCapsAdditionalFormatting,
+                (QVariant) true);
         dropCapsFormatRange.start = 0;
         dropCapsFormatRange.length = dropCapsLength + firstNonSpace;
         formatRanges.append(dropCapsFormatRange);
         layout->setAdditionalFormats(formatRanges);
         // bookkeep
         m_dropCapsNChars = dropCapsLength + firstNonSpace;
-        m_dropCapsAffectedLineWidthAdjust = 0;
         m_dropCapsAffectsNMoreLines = (m_dropCapsNChars > 0) ? dropCapsLines : 0;
     } else {
         m_dropCapsNChars = 0;
@@ -711,6 +754,7 @@ void Layout::handleTableBreak(QTextTableCell &previousCell, QTextTable *table)
         QTextCursor cur = previousCell.firstCursorPosition();
         cur = table->rowStart(cur);
         m_block = cur.block().next();
+        updateFrameStack();
         kDebug(32500) << "pos" << m_data->position() << " and block position" << m_block.position() << "and shape" << shape;
         if (!m_newShape && m_block.position() > m_data->position()) {
             m_data->setEndPosition(m_block.position() - 1);
@@ -828,7 +872,10 @@ void Layout::resetPrivate()
     m_blockData = 0;
     m_newParag = true;
     m_block = m_parent->document()->begin();
+    updateFrameStack();
     m_currentMasterPage.clear();
+    m_dropCapsPositionAdjust = 0;
+    m_dropCapsAffectedLineWidthAdjust = 0;
 
     shapeNumber = 0;
     int lastPos = -1;
@@ -840,6 +887,7 @@ void Layout::resetPrivate()
             // this shape needs to be recalculated.
             data->setPosition(lastPos + 1);
             m_block = m_parent->document()->findBlock(lastPos + 1);
+            updateFrameStack();
             m_format = m_block.blockFormat();
             if (data->documentOffset() > 0)
                 m_y = data->documentOffset();
@@ -1010,7 +1058,7 @@ void Layout::drawFrame(QTextFrame *frame, QPainter *painter, const KoTextDocumen
     for (it = frame->begin(); !(it.atEnd()); ++it) {
         QTextBlock block = it.currentBlock();
         QTextTable *table = qobject_cast<QTextTable*>(it.currentFrame());
-        QTextFrame *subFrame = qobject_cast<QTextFrame*>(it.currentFrame());
+        QTextFrame *subFrame = it.currentFrame();
 
         if (table) {
             m_tableLayout.setTable(table);
@@ -1786,6 +1834,7 @@ bool Layout::previousParag()
     if (layout->lineCount() == 0) {
         m_block = m_block.previous();
         layout = m_block.layout();
+        updateFrameStack();
     }
     QTextLine tl = layout->lineAt(0);
     Q_ASSERT(tl.isValid());
@@ -1842,17 +1891,34 @@ qreal Layout::findFootnote(const QTextLine &line, int *oldLength)
     while (pos >= 0 && pos <= line.textStart() + line.textLength()) {
         QTextCursor c1(m_block);
         c1.setPosition(m_block.position() + pos);
+        pos = text.indexOf(QChar(0xFFFC), pos + 1);
         c1.setPosition(c1.position() + 1, QTextCursor::KeepAnchor);
         KoInlineNote *note = dynamic_cast<KoInlineNote*>(m_parent->inlineTextObjectManager()->inlineTextObject(c1));
         if (note && note->type() == KoInlineNote::Footnote) {
+            QTextBlock footnoteBlock = m_textShape->footnoteDocument()->begin();
+            bool alreadyPresent = false;
+            while (footnoteBlock.isValid()) {
+                if (footnoteBlock.blockFormat().property(13471293).value<void*>() == note) {
+                    alreadyPresent = true;
+                    break;
+                }
+                footnoteBlock = footnoteBlock.next();
+            }
+            if (alreadyPresent)
+                continue;
             QTextCursor cursor(m_textShape->footnoteDocument());
-            cursor.setPosition(m_textShape->footnoteDocument()->characterCount()-1);
+            cursor.movePosition(QTextCursor::End);
             if (firstFootnote) {
                 (*oldLength) = cursor.position();
                 firstFootnote = false;
             }
             if (cursor.position() > 1)
-                cursor.insertText("\n");
+                cursor.insertBlock();
+            QTextBlockFormat bf;
+            QVariant variant;
+            variant.setValue<void*>(note);
+            bf.setProperty(13471293, variant);
+            cursor.mergeBlockFormat(bf);
 
             QTextCharFormat cf;
             cf.setVerticalAlignment(QTextCharFormat::AlignSuperScript);
@@ -1862,7 +1928,6 @@ qreal Layout::findFootnote(const QTextLine &line, int *oldLength)
             cursor.mergeCharFormat(cf);
             cursor.insertFragment(note->text());
         }
-        pos = text.indexOf(QChar(0xFFFC), pos + 1);
     }
     if (m_textShape->hasFootnoteDocument())
         return m_textShape->footnoteDocument()->size().height();
@@ -1874,3 +1939,67 @@ QTextTableCell Layout::hitTestTable(QTextTable *table, const QPointF &point)
     m_tableLayout.setTable(table);
     return m_tableLayout.hitTestTable(point);
 }
+
+void Layout::updateFrameStack()
+{
+    if (!m_block.isValid()) {
+        m_frameStack.clear();
+        m_frameStack.append(m_parent->document()->rootFrame());
+        return;
+    }
+    /* for each frame stack item (top first) check if the current block is out of range
+        and if so discard it.  */
+    for (int i = m_frameStack.count() -1; i >= 0; --i) {
+        QTextFrame *frame = m_frameStack.at(i);
+        if (frame->firstPosition() > m_block.position() + m_block.length()
+                || frame->lastPosition() < m_block.position()) {
+            m_frameStack.remove(i);
+        } else {
+            break;
+        }
+    }
+    if (m_frameStack.isEmpty())
+        m_frameStack.append(m_parent->document()->rootFrame());
+
+    int changedFrameFrom = m_frameStack.count();
+    /* repeatedly check the deepest nested frame childFrames() and add one if it contains our current block */
+    while (true) {
+        QTextFrame *frame = m_frameStack.last();
+        foreach (QTextFrame *child, frame->childFrames()) {
+            if (child->firstPosition() <= m_block.position() && child->lastPosition() > m_block.position()) {
+                m_frameStack.append(child);
+                break;
+            }
+        }
+        if (frame == m_frameStack.last())
+            break;
+    }
+
+    for (int i = changedFrameFrom ; i < m_frameStack.count(); ++i) {
+        QTextFrame *frame = m_frameStack.at(i);
+        QTextFrameFormat ff = frame->frameFormat();
+        if (ff.hasProperty(KoText::TableOfContents) && ff.property(KoText::TableOfContents).toBool() == true) {
+            // this frame is a TOC
+            bool found = false;
+            QList<QWeakPointer<ToCGenerator> >::Iterator iter = m_tocGenerators.begin();
+            while (iter != m_tocGenerators.end()) {
+                QWeakPointer<ToCGenerator> item = *iter;
+                if (item.isNull()) {
+                    iter = m_tocGenerators.erase(iter);
+                    continue;
+                }
+                if (item.data()->tocFrame() == frame) {
+                    found = true;
+                    break;
+                }
+            }
+            if (!found) {
+                ToCGenerator *tg = new ToCGenerator(frame);
+                m_tocGenerators.append(QWeakPointer<ToCGenerator>(tg));
+                // connect to FinishedLayout
+                QObject::connect(m_parent, SIGNAL(finishedLayout()),
+                        tg, SLOT(documentLayoutFinished()));
+            }
+        }
+    }
+}
