@@ -1,6 +1,6 @@
 /*
-  Copyright 2008 Brad Hards  <bradh@frogmouth.net>
-  Copyright 2009 Inge Wallin <inge@lysator.liu.se>
+  Copyright 2008        Brad Hards  <bradh@frogmouth.net>
+  Copyright 2009 - 2010 Inge Wallin <inge@lysator.liu.se>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -32,6 +32,7 @@ namespace Libemf
 
 
 OutputPainterStrategy::OutputPainterStrategy() :
+    m_header( 0 ),
     m_path( 0 ), 
     m_currentlyBuildingPath( false ), 
     m_image( 0 ), 
@@ -41,7 +42,8 @@ OutputPainterStrategy::OutputPainterStrategy() :
 
 OutputPainterStrategy::OutputPainterStrategy(QPainter &painter, QSize &size, 
                                              bool keepAspectRatio)
-    : m_path( 0 )
+    : m_header( 0 )
+    , m_path( 0 )
     , m_currentlyBuildingPath( false )
     , m_image( 0 )
     , m_currentCoords()
@@ -54,35 +56,60 @@ OutputPainterStrategy::OutputPainterStrategy(QPainter &painter, QSize &size,
 
 OutputPainterStrategy::~OutputPainterStrategy()
 {
+    delete m_header;
     //delete m_painter;
     delete m_path;
     delete m_image;
 }
 
+void OutputPainterStrategy::paintBounds(const Header *header)
+{
+    QRectF  rect(header->bounds());
+    m_painter->save();
+
+    // Draw a simple cross in a rectangle to show the bounds.
+    m_painter->setPen(QPen(QColor(172, 196, 206)));
+    m_painter->drawRect(rect);
+    m_painter->drawLine(rect.topLeft(), rect.bottomRight());
+    m_painter->drawLine(rect.bottomLeft(), rect.topRight());
+
+    m_painter->restore();
+}
+
 void OutputPainterStrategy::init( const Header *header )
 {
-    QSize  emfSize = header->bounds().size();
+    // Save the header since we need the frame and bounds inside the drawing.
+    //
+    // To be precise, it seems that the StretchDiBits record uses the
+    // physical size (stored in header.frame()) to specify where to
+    // draw the picture rather than logical coordinates.
+    m_header = new Header(*header);
 
-    //qDebug("emfOrigin  = %d, %d", header->bounds().x(), header->bounds().y());
-    //qDebug("emfSize    = %d, %d", emfSize.width(), emfSize.height() );
-    //qDebug("emfFrame   = %d, %d, %d, %d", 
-    //       header->frame().x(), header->bounds().y(),
-    //       header->bounds().width(), header->bounds().height());
-    //qDebug("emfSize    = %d, %d", emfSize.width(), emfSize.height() );
-    //qDebug("outputSize = %d, %d", m_outputSize.width(), m_outputSize.height() );
+    QSize  outputSize = header->bounds().size();
 
-    //qDebug("Device = %d, %d", header->device().width(), header->device().height() );
-    //qDebug("Millimeters = %d, %d", header->millimeters().width(), header->millimeters().height() );
+#if DEBUG_EMFPAINT
+    kDebug(31000) << "emfFrame (phys size) =" << header->frame().x() << header->frame().y()
+                  << header->frame().width() << header->frame().height();
+    kDebug(31000) << "emfBounds (log size) =" << header->bounds().x() << header->bounds().y()
+                  << header->bounds().width() << header->bounds().height();
+    kDebug(31000) << "outputSize           =" << m_outputSize.width() << m_outputSize.height();
+
+    kDebug(31000) << "Device =" << header->device().width() << header->device().height();
+    kDebug(31000) << "Millimeters =" << header->millimeters().width() 
+                  << header->millimeters().height();
+#endif
+
+#if DEBUG_EMFPAINT
+    //paintBounds(header);
+#endif
 
     // This is restored in cleanup().
     m_painter->save();
 
-    //m_painter->setPen(QColor(255,0,0));
-    //m_painter->drawRect( 0, 0, m_outputSize.width(), m_outputSize.height());// Note possible translation
     // Calculate how much the painter should be resized to fill the
     // outputSize with output.
-    qreal  scaleX = qreal( m_outputSize.width() )  / emfSize.width();
-    qreal  scaleY = qreal( m_outputSize.height() ) / emfSize.height();
+    qreal  scaleX = qreal( m_outputSize.width() )  / outputSize.width();
+    qreal  scaleY = qreal( m_outputSize.height() ) / outputSize.height();
     if ( m_keepAspectRatio ) {
         // Use the smaller value so that we don't get an overflow in
         // any direction.
@@ -90,17 +117,25 @@ void OutputPainterStrategy::init( const Header *header )
             scaleX = scaleY;
         else
             scaleY = scaleX;
-
-        // Calculate translation if we should center the Emf in the
-        // area and keep the aspect ratio.
-        m_painter->translate((m_outputSize.width() - emfSize.width() * scaleX) / 2,
-                             (m_outputSize.height() - emfSize.height() * scaleY) / 2);
     }
-    //kDebug(33100) << "scale = " << scaleX << ", " << scaleY;
-    m_painter->scale( scaleX, scaleY );
+#if DEBUG_EMFPAINT
+    kDebug(31000) << "scale = " << scaleX << ", " << scaleY;
+#endif
 
-    //m_painter->setPen(QColor(0,0,255));
-    //m_painter->drawRect( 0, 0, emfSize.width(), emfSize.height());
+    // Transform the EMF object so that it fits in the shape.
+    m_painter->scale( scaleX, scaleY );
+    m_painter->translate(-header->bounds().left(), -header->bounds().top());
+
+    // Calculate translation if we should center the Emf in the
+    // area and keep the aspect ratio.
+    if ( m_keepAspectRatio ) {
+        m_painter->translate((m_outputSize.width() - outputSize.width() * scaleX) / 2,
+                             (m_outputSize.height() - outputSize.height() * scaleY) / 2);
+    }
+
+#if DEBUG_EMFPAINT
+    paintBounds(header);
+#endif
 }
 
 void OutputPainterStrategy::cleanup( const Header *header )
@@ -126,6 +161,10 @@ void OutputPainterStrategy::setPixelV( QPoint &point, quint8 red, quint8 green,
 {
     Q_UNUSED( reserved );
 
+#if DEBUG_EMFPAINT
+    kDebug(31000) << point << red << green << blue;
+#endif
+
     m_painter->save();
 
     QPen pen;
@@ -144,6 +183,10 @@ QImage* OutputPainterStrategy::image()
    
 void OutputPainterStrategy::beginPath()
 {
+#if DEBUG_EMFPAINT
+    kDebug(31000);
+#endif
+
     delete( m_path );
     m_path = new QPainterPath;
     m_currentlyBuildingPath = true;
@@ -151,23 +194,39 @@ void OutputPainterStrategy::beginPath()
 
 void OutputPainterStrategy::closeFigure()
 {
+#if DEBUG_EMFPAINT
+    kDebug(31000);
+#endif
+
     m_path->closeSubpath();
 }
 
 void OutputPainterStrategy::endPath()
 {
+#if DEBUG_EMFPAINT
+    kDebug(31000);
+#endif
+
     m_path->setFillRule( m_fillRule );
     m_currentlyBuildingPath = false;
 }
 
 void OutputPainterStrategy::saveDC()
 {
+#if DEBUG_EMFPAINT
+    kDebug(31000);
+#endif
+
     m_painter->save();
     ++m_painterSaves;
 }
 
 void OutputPainterStrategy::restoreDC( const qint32 savedDC )
 {
+#if DEBUG_EMFPAINT
+    kDebug(31000) << savedDC;
+#endif
+
     for (int i = 0; i < savedDC; ++i) {
         if (m_painterSaves > 0) {
             m_painter->restore();
@@ -186,24 +245,40 @@ void OutputPainterStrategy::setMetaRgn()
 
 void OutputPainterStrategy::setWindowOrgEx( const QPoint &origin )
 {
+#if DEBUG_EMFPAINT
+    kDebug(31000) << origin;
+#endif
+    return;
     QSize windowSize = m_painter->window().size();
     m_painter->setWindow( QRect( origin, windowSize ) );
 }
 
 void OutputPainterStrategy::setWindowExtEx( const QSize &size )
 {
+#if DEBUG_EMFPAINT
+    kDebug(31000) << size;
+#endif
+    return;
     QPoint windowOrigin = m_painter->window().topLeft();
     m_painter->setWindow( QRect( windowOrigin, size ) );
 }
 
 void OutputPainterStrategy::setViewportOrgEx( const QPoint &origin )
 {
+#if DEBUG_EMFPAINT
+    kDebug(31000) << origin;
+#endif
+
     QSize viewportSize = m_painter->viewport().size();
     m_painter->setViewport( QRect( origin, viewportSize ) );
 }
 
 void OutputPainterStrategy::setViewportExtEx( const QSize &size )
 {
+#if DEBUG_EMFPAINT
+    kDebug(31000) << size;
+#endif
+
     QPoint viewportOrigin = m_painter->viewport().topLeft();
     m_painter->setViewport( QRect( viewportOrigin, size ) );
 }
@@ -214,6 +289,11 @@ void OutputPainterStrategy::createPen( quint32 ihPen, quint32 penStyle, quint32
     Q_UNUSED( y );
     Q_UNUSED( reserved );
 
+#if DEBUG_EMFPAINT
+    kDebug(31000) << ihPen << hex << penStyle << dec << x << y
+                  << red << green << blue << reserved;
+#endif
+
     QPen pen;
     pen.setColor( QColor( red, green, blue ) );
 
@@ -247,16 +327,16 @@ void OutputPainterStrategy::createPen( quint32 ihPen, quint32 penStyle, quint32
         pen.setStyle( Qt::SolidLine );
         break;
     case PS_USERSTYLE:
-        kDebug(33100) << "UserStyle pen not yet supported";
-        Q_ASSERT( 0 );
+        kDebug(33100) << "UserStyle pen not yet supported, using SolidLine";
+        pen.setStyle( Qt::SolidLine );
         break;
     case PS_ALTERNATE:
-        kDebug(33100) << "Alternate pen not yet supported";
-        Q_ASSERT( 0 );
+        kDebug(33100) << "Alternate pen not yet supported, using DashLine";
+        pen.setStyle( Qt::DashLine );
         break;
     default:
-        kDebug(33100) << "unexpected pen type" << (penStyle & 0xF);
-        Q_ASSERT( 0 );
+        kDebug(33100) << "unexpected pen type, using SolidLine" << (penStyle & 0xF);
+        pen.setStyle( Qt::SolidLine );
     }
    
     switch ( penStyle & PS_ENDCAP_FLAT ) {
@@ -270,8 +350,8 @@ void OutputPainterStrategy::createPen( quint32 ihPen, quint32 penStyle, quint32
         pen.setCapStyle( Qt::FlatCap );
         break;
     default:
-        kDebug(33100) << "unexpected cap style" << (penStyle & PS_ENDCAP_FLAT);
-        Q_ASSERT( 0 ); 
+        kDebug(33100) << "unexpected cap style, using SquareCap" << (penStyle & PS_ENDCAP_FLAT);
+        pen.setCapStyle( Qt::SquareCap );
     }
     pen.setWidth( x );
 
@@ -286,6 +366,11 @@ void OutputPainterStrategy:: createBrushIndirect( quint32 ihBrush, quint32 brush
     Q_UNUSED( reserved );
     Q_UNUSED( brushHatch );
 
+#if DEBUG_EMFPAINT
+    kDebug(31000) << ihBrush << hex << brushStyle << dec
+                  << red << green << blue << reserved << brushHatch;
+#endif
+
     QBrush brush;
 
     switch ( brushStyle ) {
@@ -330,23 +415,6 @@ void OutputPainterStrategy:: createBrushIndirect( quint32 ihBrush, quint32 brush
     m_objectTable.insert( ihBrush, brush );
 }
 
-int OutputPainterStrategy::convertFontWeight( quint32 emfWeight )
-{
-    if ( emfWeight == 0 ) {
-        return QFont::Normal;
-    } else if ( emfWeight <= 200 ) {
-        return QFont::Light;
-    } else if ( emfWeight <= 450 ) {
-        return QFont::Normal;
-    } else if ( emfWeight <= 650 ) {
-        return QFont::DemiBold;
-    } else if ( emfWeight <= 850 ) {
-        return QFont::Bold;
-    } else {
-        return QFont::Black;
-    }
-}
-
 void OutputPainterStrategy::extCreateFontIndirectW( const ExtCreateFontIndirectWRecord &extCreateFontIndirectW )
 {
     QFont font( extCreateFontIndirectW.fontFace() );
@@ -466,50 +534,64 @@ void OutputPainterStrategy::deleteObject( const quint32 ihObject )
 
 void OutputPainterStrategy::arc( const QRect &box, const QPoint &start, const QPoint &end )
 {
+#if DEBUG_EMFPAINT
+    kDebug(31000) << box << start << end;
+#endif
+
     QPoint centrePoint = box.center();
 
     qreal startAngle = angleFromArc( centrePoint, start );
-
-    qreal endAngle = angleFromArc( centrePoint, end );
-
-    qreal spanAngle = angularSpan( startAngle, endAngle );
+    qreal endAngle   = angleFromArc( centrePoint, end );
+    qreal spanAngle  = angularSpan( startAngle, endAngle );
 
     m_painter->drawArc( box, startAngle*16, spanAngle*16 );
 }
 
 void OutputPainterStrategy::chord( const QRect &box, const QPoint &start, const QPoint &end )
 {
+#if DEBUG_EMFPAINT
+    kDebug(31000) << box << start << end;
+#endif
+
     QPoint centrePoint = box.center();
 
     qreal startAngle = angleFromArc( centrePoint, start );
-
-    qreal endAngle = angleFromArc( centrePoint, end );
-
-    qreal spanAngle = angularSpan( startAngle, endAngle );
+    qreal endAngle   = angleFromArc( centrePoint, end );
+    qreal spanAngle  = angularSpan( startAngle, endAngle );
 
     m_painter->drawChord( box, startAngle*16, spanAngle*16 ); 
 }
 
 void OutputPainterStrategy::pie( const QRect &box, const QPoint &start, const QPoint &end )
 {
+#if DEBUG_EMFPAINT
+    kDebug(31000) << box << start << end;
+#endif
+
     QPoint centrePoint = box.center();
 
     qreal startAngle = angleFromArc( centrePoint, start );
-
-    qreal endAngle = angleFromArc( centrePoint, end );
-
-    qreal spanAngle = angularSpan( startAngle, endAngle );
+    qreal endAngle   = angleFromArc( centrePoint, end );
+    qreal spanAngle  = angularSpan( startAngle, endAngle );
 
     m_painter->drawPie( box, startAngle*16, spanAngle*16 );
 }
 
 void OutputPainterStrategy::ellipse( const QRect &box )
 {
+#if DEBUG_EMFPAINT
+    kDebug(31000) << box;
+#endif
+
     m_painter->drawEllipse( box );
 }
 
 void OutputPainterStrategy::rectangle( const QRect &box )
 {
+#if DEBUG_EMFPAINT
+    kDebug(31000) << box;
+#endif
+
     m_painter->drawRect( box );
 }
 
@@ -520,6 +602,10 @@ void OutputPainterStrategy::setMapMode( const quint32 mapMode )
 
 void OutputPainterStrategy::setBkMode( const quint32 backgroundMode )
 {
+#if DEBUG_EMFPAINT
+    kDebug(31000) << backgroundMode;
+#endif
+
     if ( backgroundMode == TRANSPARENT ) {
         m_painter->setBackgroundMode( Qt::TransparentMode );
     } else if ( backgroundMode == OPAQUE ) {
@@ -532,6 +618,10 @@ void OutputPainterStrategy::setBkMode( const quint32 backgroundMode )
 
 void OutputPainterStrategy::setPolyFillMode( const quint32 polyFillMode )
 {
+#if DEBUG_EMFPAINT
+    kDebug(31000) << polyFillMode;
+#endif
+
     if ( polyFillMode == ALTERNATE ) {
 	m_fillRule = Qt::OddEvenFill;
     } else if ( polyFillMode == WINDING ) {
@@ -544,6 +634,10 @@ void OutputPainterStrategy::setPolyFillMode( const quint32 polyFillMode )
 
 void OutputPainterStrategy::setLayout( const quint32 layoutMode )
 {
+#if DEBUG_EMFPAINT
+    kDebug(31000) << layoutMode;
+#endif
+
     if ( layoutMode == LAYOUT_LTR ) {
         m_painter->setLayoutDirection( Qt::LeftToRight );
     } else if ( layoutMode == LAYOUT_RTL ) {
@@ -556,6 +650,10 @@ void OutputPainterStrategy::setLayout( const quint32 layoutMode )
 
 void OutputPainterStrategy::setTextAlign( const quint32 textAlignMode )
 {
+#if DEBUG_EMFPAINT
+    kDebug(31000) << textAlignMode;
+#endif
+
     m_textAlignMode = textAlignMode;
 }
 
@@ -564,6 +662,10 @@ void OutputPainterStrategy::setTextColor( const quint8 red, const quint8 green,
 {
     Q_UNUSED( reserved );
 
+#if DEBUG_EMFPAINT
+    kDebug(31000) << red << green << blue << reserved;
+#endif
+
     m_textPen.setColor( QColor( red, green, blue ) );
 }
 
@@ -572,12 +674,20 @@ void OutputPainterStrategy::setBkColor( const quint8 red, const quint8 green, co
 {
     Q_UNUSED( reserved );
 
+#if DEBUG_EMFPAINT
+    kDebug(31000) << red << green << blue << reserved;
+#endif
+
     m_painter->setBackground( QBrush( QColor( red, green, blue ) ) );
 }
 
 void OutputPainterStrategy::modifyWorldTransform( const quint32 mode, float M11, float M12,
                                                   float M21, float M22, float Dx, float Dy )
 {
+#if DEBUG_EMFPAINT
+    kDebug(31000) << mode << M11 << M12 << M21 << M22 << Dx << Dy;
+#endif
+
     QMatrix matrix( M11, M12, M21, M22, Dx, Dy);
 
     if ( mode == MWT_IDENTITY ) {
@@ -598,6 +708,10 @@ void OutputPainterStrategy::modifyWorldTransform( const quint32 mode, float M11,
 void OutputPainterStrategy::setWorldTransform( float M11, float M12, float M21,
                                                float M22, float Dx, float Dy )
 {
+#if DEBUG_EMFPAINT
+    kDebug(31000) << M11 << M12 << M21 << M22 << Dx << Dy;
+#endif
+
     QMatrix matrix( M11, M12, M21, M22, Dx, Dy);
 
     m_painter->setWorldMatrix( matrix );
@@ -605,6 +719,10 @@ void OutputPainterStrategy::setWorldTransform( float M11, float M12, float M21,
 
 void OutputPainterStrategy::extTextOutA( const ExtTextOutARecord &extTextOutA )
 {
+#if DEBUG_EMFPAINT
+    //kDebug(31000) << extTextOutA;  FIXME
+#endif
+
     m_painter->save();
 
     m_painter->setPen( m_textPen );
@@ -635,82 +753,91 @@ void OutputPainterStrategy::extTextOutA( const ExtTextOutARecord &extTextOutA )
     m_painter->restore();
 }
 
-void OutputPainterStrategy::extTextOutW( const QPoint &referencePoint, const QString &textString )
+void OutputPainterStrategy::extTextOutW( const QPoint &referencePoint, const QString &text )
 {
-    m_painter->save();
+#if DEBUG_EMFPAINT
+    kDebug(31000) << referencePoint << text;
+#endif
 
-    m_painter->setPen( m_textPen );
-    m_painter->drawText( referencePoint, textString );
+    int  x = referencePoint.x();
+    int  y = referencePoint.y();
 
-    kDebug(33100) << "extTextOutW: ref.point = " << referencePoint.x() << " " << referencePoint.y()
-                  << " Text = " << textString.toLatin1().data();
+    // The TA_UPDATECP flag tells us to use the current position
+    if (m_textAlignMode & TA_UPDATECP) {
+        // (left, top) position = current logical position
+        x = m_currentCoords.x();
+        y = m_currentCoords.y();
+    }
 
-    m_painter->restore();
+    QFontMetrics  fm = m_painter->fontMetrics();
+    int width  = fm.width(text) + fm.descent();    // fm.width(text) isn't right with Italic text
+    int height = fm.height();
+
+    // Horizontal align.  These flags are supposed to be mutually exclusive.
+    if ((m_textAlignMode & TA_CENTER) == TA_CENTER)
+        x -= (width / 2);
+    else if ((m_textAlignMode & TA_RIGHT) == TA_RIGHT)
+        x -= width;
+
+    // Vertical align. 
+    if ((m_textAlignMode & TA_BASELINE) == TA_BASELINE)
+        y -= fm.ascent();  // (height - fm.descent()) is used in qwmf.  This should be the same.
+    else if ((m_textAlignMode & TA_BOTTOM) == TA_BOTTOM) {
+        y -= height;
+
+#if DEBUG_EMFPAINT
+        kDebug(31000) << "font = " << m_painter->font() << " pointSize = " << m_painter->font().pointSize()
+                      << "ascent = " << fm.ascent() << " height = " << fm.height()
+                      << "leading = " << fm.leading();
+#endif
+    }
+
+    // Use the special pen defined by mTextPen for text.
+    QPen  savePen = m_painter->pen();
+    m_painter->setPen(m_textPen);
+
+    m_painter->drawText(x, y, width, height, Qt::AlignLeft|Qt::AlignTop, text);
+
+    m_painter->setPen(savePen);
 }
 
 void OutputPainterStrategy::moveToEx( const quint32 x, const quint32 y )
 {
-#if 1
-    if ( currentlyBuildingPath() )
+#if DEBUG_EMFPAINT
+    kDebug(31000) << x << y;
+#endif
+
+    if ( m_currentlyBuildingPath )
         m_path->moveTo( QPoint( x, y ) );
     else
         m_currentCoords = QPoint( x, y );
-
-#else
-    // This is what the code looked before.  I think the
-    // interpretation above is the correct one.
-    if ( ! currentlyBuildingPath() ) {
-        beginPath();
-    }
-    m_path->moveTo( QPoint( x, y ) );
-#endif
 }
 
 void OutputPainterStrategy::lineTo( const QPoint &finishPoint )
 {
-#if 1
-    if ( currentlyBuildingPath() )
+#if DEBUG_EMFPAINT
+    kDebug(31000) << finishPoint;
+#endif
+
+    if ( m_currentlyBuildingPath )
         m_path->lineTo( finishPoint );
     else {
         m_painter->drawLine( m_currentCoords, finishPoint );
         m_currentCoords = finishPoint;
     }
-
-#else
-    // This is what the code looked before.  I think the
-    // interpretation above is the correct one.
-    m_path->lineTo( finishPoint );
-#endif
-}
-
-qreal OutputPainterStrategy::angleFromArc( const QPoint &centrePoint, const QPoint &radialPoint )
-{
-    double dX = radialPoint.x() - centrePoint.x();
-    double dY = centrePoint.y() - radialPoint.y();
-    // Qt angles are in degrees. atan2 returns radians
-    return ( atan2( dY, dX ) * 180 / M_PI );
-}
-
-qreal OutputPainterStrategy::angularSpan( const qreal startAngle, const qreal endAngle )
-{
-    qreal spanAngle = endAngle - startAngle;
-    
-    if ( spanAngle <= 0 ) {
-        spanAngle += 360;
-    }
-    
-    return spanAngle;
 }
 
 void OutputPainterStrategy::arcTo( const QRect &box, const QPoint &start, const QPoint &end )
 {
+#if DEBUG_EMFPAINT
+    kDebug(31000) << box << start << end;
+#endif
+
     QPoint centrePoint = box.center();
 
     qreal startAngle = angleFromArc( centrePoint, start );
-
-    qreal endAngle = angleFromArc( centrePoint, end );
-
-    qreal spanAngle = angularSpan( startAngle, endAngle );
+    qreal endAngle   = angleFromArc( centrePoint, end );
+    qreal spanAngle  = angularSpan( startAngle, endAngle );
 
     m_path->arcTo( box, startAngle, spanAngle );
 }
@@ -718,6 +845,11 @@ void OutputPainterStrategy::arcTo( const QRect &box, const QPoint &start, const
 void OutputPainterStrategy::polygon16( const QRect &bounds, const QList<QPoint> points )
 {
     Q_UNUSED( bounds );
+
+#if DEBUG_EMFPAINT
+    kDebug(31000) << bounds << points;
+#endif
+
     QVector<QPoint> pointVector = points.toVector();
     m_painter->drawPolygon( pointVector.constData(), pointVector.size(), m_fillRule );
 }
@@ -725,12 +857,21 @@ void OutputPainterStrategy::polygon16( const QRect &bounds, const QList<QPoint>
 void OutputPainterStrategy::polyLine( const QRect &bounds, const QList<QPoint> points )
 {
     Q_UNUSED( bounds );
+
+#if DEBUG_EMFPAINT
+    kDebug(31000) << bounds << points;
+#endif
+
     QVector<QPoint> pointVector = points.toVector();
     m_painter->drawPolyline( pointVector.constData(), pointVector.size() );
 }
 
 void OutputPainterStrategy::polyLine16( const QRect &bounds, const QList<QPoint> points )
 {
+#if DEBUG_EMFPAINT
+    kDebug(31000) << bounds << points;
+#endif
+
     polyLine( bounds, points );
 }
 
@@ -738,6 +879,10 @@ void OutputPainterStrategy::polyPolygon16( const QRect &bounds, const QList< QVe
 {
     Q_UNUSED( bounds );
 
+#if DEBUG_EMFPAINT
+    kDebug(31000) << bounds << points;
+#endif
+
     for ( int i = 0; i < points.size(); ++i ) {
         m_painter->drawPolygon( points[i].constData(), points[i].size(), m_fillRule );
     }
@@ -747,6 +892,10 @@ void OutputPainterStrategy::polyPolyLine16( const QRect &bounds, const QList< QV
 {
     Q_UNUSED( bounds );
 
+#if DEBUG_EMFPAINT
+    kDebug(31000) << bounds << points;
+#endif
+
     for ( int i = 0; i < points.size(); ++i ) {
         m_painter->drawPolyline( points[i].constData(), points[i].size() );
     }
@@ -755,6 +904,11 @@ void OutputPainterStrategy::polyPolyLine16( const QRect &bounds, const QList< QV
 void OutputPainterStrategy::polyLineTo16( const QRect &bounds, const QList<QPoint> points )
 {
     Q_UNUSED( bounds );
+
+#if DEBUG_EMFPAINT
+    kDebug(31000) << bounds << points;
+#endif
+
     for ( int i = 0; i < points.count(); ++i ) {
 	m_path->lineTo( points[i] );
     }
@@ -762,6 +916,10 @@ void OutputPainterStrategy::polyLineTo16( const QRect &bounds, const QList<QPoin
 
 void OutputPainterStrategy::polyBezier16( const QRect &bounds, const QList<QPoint> points )
 {
+#if DEBUG_EMFPAINT
+    kDebug(31000) << bounds << points;
+#endif
+
     Q_UNUSED( bounds );
     QPainterPath path;
     path.moveTo( points[0] );
@@ -773,6 +931,10 @@ void OutputPainterStrategy::polyBezier16( const QRect &bounds, const QList<QPoin
 
 void OutputPainterStrategy::polyBezierTo16( const QRect &bounds, const QList<QPoint> points )
 {
+#if DEBUG_EMFPAINT
+    kDebug(31000) << bounds << points;
+#endif
+
     Q_UNUSED( bounds );
     for ( int i = 0; i < points.count(); i+=3 ) {
 	m_path->cubicTo( points[i], points[i+1], points[i+2] );
@@ -781,64 +943,88 @@ void OutputPainterStrategy::polyBezierTo16( const QRect &bounds, const QList<QPo
 
 void OutputPainterStrategy::fillPath( const QRect &bounds )
 {
+#if DEBUG_EMFPAINT
+    kDebug(31000) << bounds;
+#endif
+
     Q_UNUSED( bounds );
     m_painter->fillPath( *m_path, m_painter->brush() );
 }
 
 void OutputPainterStrategy::strokeAndFillPath( const QRect &bounds )
 {
+#if DEBUG_EMFPAINT
+    kDebug(31000) << bounds;
+#endif
+
     Q_UNUSED( bounds );
     m_painter->drawPath( *m_path );
 }
 
 void OutputPainterStrategy::strokePath( const QRect &bounds )
 {
+#if DEBUG_EMFPAINT
+    kDebug(31000) << bounds;
+#endif
+
     Q_UNUSED( bounds );
     m_painter->strokePath( *m_path, m_painter->pen() );
 }
 
 void OutputPainterStrategy::setClipPath( const quint32 regionMode )
 {
-    switch ( regionMode ) {
-        case RGN_AND:
-        {
-            m_painter->setClipPath( *m_path, Qt::IntersectClip );
-        }
-        break;
-        case RGN_OR:
-        {
-            m_painter->setClipPath( *m_path, Qt::UniteClip );
-        }
-        break;
-        case RGN_COPY:
-        {
-            m_painter->setClipPath( *m_path, Qt::ReplaceClip );
-        }
-        break;
-        default:
-        {
-            qWarning() <<  "Unexpected / unsupported clip region mode:" << regionMode; 
-            Q_ASSERT( 0 );
-        }
-    }
-}
+#if DEBUG_EMFPAINT
+    kDebug(31000) << hex << regionMode << dec;
+#endif
 
-bool OutputPainterStrategy::currentlyBuildingPath() const
-{
-    return ( m_currentlyBuildingPath );
+    switch ( regionMode ) {
+    case RGN_AND:
+        m_painter->setClipPath( *m_path, Qt::IntersectClip );
+        break;
+    case RGN_OR:
+        m_painter->setClipPath( *m_path, Qt::UniteClip );
+        break;
+    case RGN_COPY:
+        m_painter->setClipPath( *m_path, Qt::ReplaceClip );
+        break;
+    default:
+        qWarning() <<  "Unexpected / unsupported clip region mode:" << regionMode; 
+        Q_ASSERT( 0 );
+    }
 }
 
 void OutputPainterStrategy::bitBlt( BitBltRecord bitBltRecord )
 {
+#if DEBUG_EMFPAINT
+    kDebug(31000) << bitBltRecord.xDest() << bitBltRecord.yDest()
+                  << bitBltRecord.cxDest() << bitBltRecord.cyDest()
+                  << hex << bitBltRecord.rasterOperation() << dec
+                  << bitBltRecord.bkColorSrc();
+#endif
+
     QRect target( bitBltRecord.xDest(), bitBltRecord.yDest(),
                   bitBltRecord.cxDest(), bitBltRecord.cyDest() );
-    if ( bitBltRecord.hasImage() ) {
+    // 0x00f00021 is the PatCopy raster operation which just fills a rectangle with a brush.
+    // This seems to be the most common one.
+    //
+    // FIXME: Implement the rest of the raster operations.
+    if (bitBltRecord.rasterOperation() == 0x00f00021) {
+        // Would have been nice if we didn't have to pull out the
+        // brush to use it with fillRect()...
+        QBrush brush = m_painter->brush();
+        m_painter->fillRect(target, brush);
+    }
+    else if ( bitBltRecord.hasImage() ) {
         m_painter->drawImage( target, *(bitBltRecord.image()) );
     }
 }
 
 void OutputPainterStrategy::setStretchBltMode( const quint32 stretchMode )
 {
+#if DEBUG_EMFPAINT
+    kDebug(31000) << hex << stretchMode << dec;
+#endif
+
     switch ( stretchMode ) {
     case 0x01:
         kDebug(33100) << "EMR_STRETCHBLTMODE: STRETCH_ANDSCANS";
@@ -857,17 +1043,23 @@ void OutputPainterStrategy::setStretchBltMode( const quint32 stretchMode )
     }
 }
 
-void OutputPainterStrategy::stretchDiBits( StretchDiBitsRecord stretchDiBitsRecord )
+void OutputPainterStrategy::stretchDiBits( StretchDiBitsRecord record )
 {
-    QPoint targetPosition( stretchDiBitsRecord.xDest(),
-                           stretchDiBitsRecord.yDest() );
-    QSize targetSize( stretchDiBitsRecord.cxDest(),
-                      stretchDiBitsRecord.cyDest() );
+#if DEBUG_EMFPAINT
+    kDebug(31000) << "Bounds:    " << record.bounds();
+    kDebug(31000) << "Dest rect: " 
+                  << record.xDest() << record.yDest() << record.cxDest() << record.cyDest();
+    kDebug(31000) << "Src rect:  " 
+                  << record.xSrc() << record.ySrc() << record.cxSrc() << record.cySrc();
+    kDebug(31000) << "Raster op: " << hex << record.rasterOperation() << dec;
+                  //<< record.bkColorSrc();
+#endif
 
-    QPoint sourcePosition( stretchDiBitsRecord.xSrc(),
-                           stretchDiBitsRecord.ySrc() );
-    QSize sourceSize( stretchDiBitsRecord.cxSrc(),
-                      stretchDiBitsRecord.cySrc() );
+    QPoint targetPosition( record.xDest(), record.yDest() );
+    QSize  targetSize( record.cxDest(), record.cyDest() );
+
+    QPoint sourcePosition( record.xSrc(), record.ySrc() );
+    QSize  sourceSize( record.cxSrc(), record.cySrc() );
 
     // special cases, from [MS-EMF] Section 2.3.1.7:
     // "This record specifies a mirror-image copy of the source bitmap to the
@@ -898,7 +1090,69 @@ void OutputPainterStrategy::stretchDiBits( StretchDiBitsRecord stretchDiBitsReco
         targetPosition.ry() -= targetSize.height();
         target = QRect( targetPosition, targetSize );
     }
-    m_painter->drawImage( target, *(stretchDiBitsRecord.image()), source );
+
+    // SRCCOPY is the simplest case.  TODO: implement the rest.
+    if (record.rasterOperation() == 0x00cc0020) {
+#if DEBUG_EMFPAINT
+        kDebug(31000) << "target" << target;
+        kDebug(31000) << "source" << source;
+        kDebug(31000) << "image" << record.image()->size();
+#endif
+        // For some reason, the target coordinates for the picture are
+        // expressed in physical coordinates (Frame in the header)
+        // instead of logical coordinates like all the other types of
+        // records.  Therefore we have to rescale the target from
+        // physical to logical coordinates.
+        qreal scaleX = qreal(m_header->frame().width()) / qreal(m_header->bounds().width());
+        qreal scaleY = qreal(m_header->frame().height()) / qreal(m_header->bounds().height());
+        QRect realTarget(QPoint(target.x() / scaleX, target.y() / scaleY),
+                         QSize(target.width() / scaleX, target.height() / scaleY));
+        m_painter->drawImage(realTarget, *(record.image()), source);
+        //m_painter->drawImage(target, *(record.image()), source);
+    }
+}
+
+
+// ----------------------------------------------------------------
+//                         Private functions
+
+
+qreal OutputPainterStrategy::angleFromArc( const QPoint &centrePoint, const QPoint &radialPoint )
+{
+    double dX = radialPoint.x() - centrePoint.x();
+    double dY = centrePoint.y() - radialPoint.y();
+    // Qt angles are in degrees. atan2 returns radians
+    return ( atan2( dY, dX ) * 180 / M_PI );
+}
+
+qreal OutputPainterStrategy::angularSpan( const qreal startAngle, const qreal endAngle )
+{
+    qreal spanAngle = endAngle - startAngle;
+    
+    if ( spanAngle <= 0 ) {
+        spanAngle += 360;
+    }
+    
+    return spanAngle;
+}
+
+int OutputPainterStrategy::convertFontWeight( quint32 emfWeight )
+{
+    // FIXME: See how it's done in the wmf library and check if this is suitable here.
+
+    if ( emfWeight == 0 ) {
+        return QFont::Normal;
+    } else if ( emfWeight <= 200 ) {
+        return QFont::Light;
+    } else if ( emfWeight <= 450 ) {
+        return QFont::Normal;
+    } else if ( emfWeight <= 650 ) {
+        return QFont::DemiBold;
+    } else if ( emfWeight <= 850 ) {
+        return QFont::Bold;
+    } else {
+        return QFont::Black;
+    }
 }
 
 } // xnamespace...
